@@ -14,6 +14,7 @@ use Doctrine\ORM\UnitOfWork;
 use function array_fill_keys;
 use function array_keys;
 use function array_map;
+use function assert;
 use function count;
 use function is_array;
 use function key;
@@ -28,36 +29,32 @@ use function spl_object_id;
 class ObjectHydrator extends AbstractHydrator
 {
     /** @var mixed[] */
-    private $identifierMap = [];
+    private array $identifierMap = [];
 
     /** @var mixed[] */
-    private $resultPointers = [];
+    private array $resultPointers = [];
 
     /** @var mixed[] */
-    private $idTemplate = [];
+    private array $idTemplate = [];
 
-    /** @var int */
-    private $resultCounter = 0;
+    private int $resultCounter = 0;
 
     /** @var mixed[] */
-    private $rootAliases = [];
+    private array $rootAliases = [];
 
     /** @var mixed[] */
-    private $initializedCollections = [];
+    private array $initializedCollections = [];
 
     /** @var array<string, PersistentCollection> */
-    private $uninitializedCollections = [];
+    private array $uninitializedCollections = [];
 
     /** @var mixed[] */
-    private $existingCollections = [];
+    private array $existingCollections = [];
 
-    /**
-     * {@inheritDoc}
-     */
-    protected function prepare()
+    protected function prepare(): void
     {
-        if (! isset($this->_hints[UnitOfWork::HINT_DEFEREAGERLOAD])) {
-            $this->_hints[UnitOfWork::HINT_DEFEREAGERLOAD] = true;
+        if (! isset($this->hints[UnitOfWork::HINT_DEFEREAGERLOAD])) {
+            $this->hints[UnitOfWork::HINT_DEFEREAGERLOAD] = true;
         }
 
         foreach ($this->resultSetMapping()->aliasMap as $dqlAlias => $className) {
@@ -80,39 +77,36 @@ class ObjectHydrator extends AbstractHydrator
             $sourceClass     = $this->getClassMetadata($sourceClassName);
             $assoc           = $sourceClass->associationMappings[$this->resultSetMapping()->relationMap[$dqlAlias]];
 
-            $this->_hints['fetched'][$parent][$assoc['fieldName']] = true;
+            $this->hints['fetched'][$parent][$assoc->fieldName] = true;
 
-            if ($assoc['type'] === ClassMetadata::MANY_TO_MANY) {
+            if ($assoc->isManyToMany()) {
                 continue;
             }
 
             // Mark any non-collection opposite sides as fetched, too.
-            if ($assoc['mappedBy']) {
-                $this->_hints['fetched'][$dqlAlias][$assoc['mappedBy']] = true;
+            if (! $assoc->isOwningSide()) {
+                $this->hints['fetched'][$dqlAlias][$assoc->mappedBy] = true;
 
                 continue;
             }
 
             // handle fetch-joined owning side bi-directional one-to-one associations
-            if ($assoc['inversedBy']) {
+            if ($assoc->inversedBy !== null) {
                 $class        = $this->getClassMetadata($className);
-                $inverseAssoc = $class->associationMappings[$assoc['inversedBy']];
+                $inverseAssoc = $class->associationMappings[$assoc->inversedBy];
 
-                if (! ($inverseAssoc['type'] & ClassMetadata::TO_ONE)) {
+                if (! $inverseAssoc->isToOne()) {
                     continue;
                 }
 
-                $this->_hints['fetched'][$dqlAlias][$inverseAssoc['fieldName']] = true;
+                $this->hints['fetched'][$dqlAlias][$inverseAssoc->fieldName] = true;
             }
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    protected function cleanup()
+    protected function cleanup(): void
     {
-        $eagerLoad = isset($this->_hints[UnitOfWork::HINT_DEFEREAGERLOAD]) && $this->_hints[UnitOfWork::HINT_DEFEREAGERLOAD] === true;
+        $eagerLoad = isset($this->hints[UnitOfWork::HINT_DEFEREAGERLOAD]) && $this->hints[UnitOfWork::HINT_DEFEREAGERLOAD] === true;
 
         parent::cleanup();
 
@@ -123,10 +117,10 @@ class ObjectHydrator extends AbstractHydrator
         $this->resultPointers           = [];
 
         if ($eagerLoad) {
-            $this->_uow->triggerEagerLoads();
+            $this->uow->triggerEagerLoads();
         }
 
-        $this->_uow->hydrationComplete();
+        $this->uow->hydrationComplete();
     }
 
     protected function cleanupAfterRowIteration(): void
@@ -141,7 +135,7 @@ class ObjectHydrator extends AbstractHydrator
     /**
      * {@inheritDoc}
      */
-    protected function hydrateAllData()
+    protected function hydrateAllData(): array
     {
         $result = [];
 
@@ -166,15 +160,14 @@ class ObjectHydrator extends AbstractHydrator
     /**
      * Initializes a related collection.
      *
-     * @param object $entity         The entity to which the collection belongs.
      * @param string $fieldName      The name of the field on the entity that holds the collection.
      * @param string $parentDqlAlias Alias of the parent fetch joining this collection.
      */
     private function initRelatedCollection(
-        $entity,
+        object $entity,
         ClassMetadata $class,
         string $fieldName,
-        string $parentDqlAlias
+        string $parentDqlAlias,
     ): PersistentCollection {
         $oid      = spl_object_id($entity);
         $relation = $class->associationMappings[$fieldName];
@@ -185,20 +178,21 @@ class ObjectHydrator extends AbstractHydrator
         }
 
         if (! $value instanceof PersistentCollection) {
+            assert($relation->isToMany());
             $value = new PersistentCollection(
-                $this->_em,
-                $this->_metadataCache[$relation['targetEntity']],
-                $value
+                $this->em,
+                $this->metadataCache[$relation->targetEntity],
+                $value,
             );
             $value->setOwner($entity, $relation);
 
             $class->reflFields[$fieldName]->setValue($entity, $value);
-            $this->_uow->setOriginalEntityProperty($oid, $fieldName, $value);
+            $this->uow->setOriginalEntityProperty($oid, $fieldName, $value);
 
             $this->initializedCollections[$oid . $fieldName] = $value;
         } elseif (
-            isset($this->_hints[Query::HINT_REFRESH]) ||
-            isset($this->_hints['fetched'][$parentDqlAlias][$fieldName]) &&
+            isset($this->hints[Query::HINT_REFRESH]) ||
+            isset($this->hints['fetched'][$parentDqlAlias][$fieldName]) &&
              ! $value->isInitialized()
         ) {
             // Is already PersistentCollection, but either REFRESH or FETCH-JOIN and UNINITIALIZED!
@@ -219,13 +213,11 @@ class ObjectHydrator extends AbstractHydrator
      * Gets an entity instance.
      *
      * @param string $dqlAlias The DQL alias of the entity's class.
-     * @psalm-param array<string, mixed> $data     The instance data.
-     *
-     * @return object
+     * @phpstan-param array<string, mixed> $data     The instance data.
      *
      * @throws HydrationException
      */
-    private function getEntity(array $data, string $dqlAlias)
+    private function getEntity(array $data, string $dqlAlias): object
     {
         $className = $this->resultSetMapping()->aliasMap[$dqlAlias];
 
@@ -246,7 +238,7 @@ class ObjectHydrator extends AbstractHydrator
                 throw HydrationException::emptyDiscriminatorValue($dqlAlias);
             }
 
-            $discrMap           = $this->_metadataCache[$className]->discriminatorMap;
+            $discrMap           = $this->metadataCache[$className]->discriminatorMap;
             $discriminatorValue = $data[$discrColumn];
             if ($discriminatorValue instanceof BackedEnum) {
                 $discriminatorValue = $discriminatorValue->value;
@@ -263,45 +255,44 @@ class ObjectHydrator extends AbstractHydrator
             unset($data[$discrColumn]);
         }
 
-        if (isset($this->_hints[Query::HINT_REFRESH_ENTITY], $this->rootAliases[$dqlAlias])) {
-            $this->registerManaged($this->_metadataCache[$className], $this->_hints[Query::HINT_REFRESH_ENTITY], $data);
+        if (isset($this->hints[Query::HINT_REFRESH_ENTITY], $this->rootAliases[$dqlAlias])) {
+            $this->registerManaged($this->metadataCache[$className], $this->hints[Query::HINT_REFRESH_ENTITY], $data);
         }
 
-        $this->_hints['fetchAlias'] = $dqlAlias;
+        $this->hints['fetchAlias'] = $dqlAlias;
 
-        return $this->_uow->createEntity($className, $data, $this->_hints);
+        return $this->uow->createEntity($className, $data, $this->hints);
     }
 
     /**
-     * @psalm-param class-string $className
-     * @psalm-param array<string, mixed> $data
-     *
-     * @return mixed
+     * @param class-string $className
+     * @phpstan-param array<string, mixed> $data
      */
-    private function getEntityFromIdentityMap(string $className, array $data)
+    private function getEntityFromIdentityMap(string $className, array $data): object|bool
     {
         // TODO: Abstract this code and UnitOfWork::createEntity() equivalent?
-        $class = $this->_metadataCache[$className];
+        $class = $this->metadataCache[$className];
 
         if ($class->isIdentifierComposite) {
             $idHash = UnitOfWork::getIdHashByIdentifier(
                 array_map(
                     /** @return mixed */
-                    static function (string $fieldName) use ($data, $class) {
-                        return isset($class->associationMappings[$fieldName])
-                            ? $data[$class->associationMappings[$fieldName]['joinColumns'][0]['name']]
-                            : $data[$fieldName];
-                    },
-                    $class->identifier
-                )
+                    static fn (string $fieldName) => isset($class->associationMappings[$fieldName]) && assert($class->associationMappings[$fieldName]->isToOneOwningSide())
+                        ? $data[$class->associationMappings[$fieldName]->joinColumns[0]->name]
+                        : $data[$fieldName],
+                    $class->identifier,
+                ),
             );
 
-            return $this->_uow->tryGetByIdHash(ltrim($idHash), $class->rootEntityName);
+            return $this->uow->tryGetByIdHash(ltrim($idHash), $class->rootEntityName);
         } elseif (isset($class->associationMappings[$class->identifier[0]])) {
-            return $this->_uow->tryGetByIdHash($data[$class->associationMappings[$class->identifier[0]]['joinColumns'][0]['name']], $class->rootEntityName);
+            $association = $class->associationMappings[$class->identifier[0]];
+            assert($association->isToOneOwningSide());
+
+            return $this->uow->tryGetByIdHash($data[$association->joinColumns[0]->name], $class->rootEntityName);
         }
 
-        return $this->_uow->tryGetByIdHash($data[$class->identifier[0]], $class->rootEntityName);
+        return $this->uow->tryGetByIdHash($data[$class->identifier[0]], $class->rootEntityName);
     }
 
     /**
@@ -322,10 +313,8 @@ class ObjectHydrator extends AbstractHydrator
      *
      * @param mixed[] $row    The data of the row to process.
      * @param mixed[] $result The result array to fill.
-     *
-     * @return void
      */
-    protected function hydrateRowData(array $row, array &$result)
+    protected function hydrateRowData(array $row, array &$result): void
     {
         // Initialize
         $id                 = $this->idTemplate; // initialize the id-memory
@@ -354,7 +343,7 @@ class ObjectHydrator extends AbstractHydrator
                     continue;
                 }
 
-                $parentClass   = $this->_metadataCache[$this->resultSetMapping()->aliasMap[$parentAlias]];
+                $parentClass   = $this->metadataCache[$this->resultSetMapping()->aliasMap[$parentAlias]];
                 $relationField = $this->resultSetMapping()->relationMap[$dqlAlias];
                 $relation      = $parentClass->associationMappings[$relationField];
                 $reflField     = $parentClass->reflFields[$relationField];
@@ -367,21 +356,25 @@ class ObjectHydrator extends AbstractHydrator
                     $parentObject = $this->resultPointers[$parentAlias];
                 } else {
                     // Parent object of relation not found, mark as not-fetched again
-                    $element = $this->getEntity($data, $dqlAlias);
+                    if (isset($nonemptyComponents[$dqlAlias])) {
+                        $element = $this->getEntity($data, $dqlAlias);
 
-                    // Update result pointer and provide initial fetch data for parent
-                    $this->resultPointers[$dqlAlias]               = $element;
-                    $rowData['data'][$parentAlias][$relationField] = $element;
+                        // Update result pointer and provide initial fetch data for parent
+                        $this->resultPointers[$dqlAlias]               = $element;
+                        $rowData['data'][$parentAlias][$relationField] = $element;
+                    } else {
+                        $element = null;
+                    }
 
                     // Mark as not-fetched again
-                    unset($this->_hints['fetched'][$parentAlias][$relationField]);
+                    unset($this->hints['fetched'][$parentAlias][$relationField]);
                     continue;
                 }
 
                 $oid = spl_object_id($parentObject);
 
                 // Check the type of the relation (many or single-valued)
-                if (! ($relation['type'] & ClassMetadata::TO_ONE)) {
+                if (! $relation->isToOne()) {
                     // PATH A: Collection-valued association
                     $reflFieldValue = $reflField->getValue($parentObject);
 
@@ -438,35 +431,35 @@ class ObjectHydrator extends AbstractHydrator
                     // PATH B: Single-valued association
                     $reflFieldValue = $reflField->getValue($parentObject);
 
-                    if (! $reflFieldValue || isset($this->_hints[Query::HINT_REFRESH]) || $this->_uow->isUninitializedObject($reflFieldValue)) {
+                    if (! $reflFieldValue || isset($this->hints[Query::HINT_REFRESH]) || $this->uow->isUninitializedObject($reflFieldValue)) {
                         // we only need to take action if this value is null,
                         // we refresh the entity or its an uninitialized proxy.
                         if (isset($nonemptyComponents[$dqlAlias])) {
                             $element = $this->getEntity($data, $dqlAlias);
                             $reflField->setValue($parentObject, $element);
-                            $this->_uow->setOriginalEntityProperty($oid, $relationField, $element);
-                            $targetClass = $this->_metadataCache[$relation['targetEntity']];
+                            $this->uow->setOriginalEntityProperty($oid, $relationField, $element);
+                            $targetClass = $this->metadataCache[$relation->targetEntity];
 
-                            if ($relation['isOwningSide']) {
+                            if ($relation->isOwningSide()) {
                                 // TODO: Just check hints['fetched'] here?
                                 // If there is an inverse mapping on the target class its bidirectional
-                                if ($relation['inversedBy']) {
-                                    $inverseAssoc = $targetClass->associationMappings[$relation['inversedBy']];
-                                    if ($inverseAssoc['type'] & ClassMetadata::TO_ONE) {
-                                        $targetClass->reflFields[$inverseAssoc['fieldName']]->setValue($element, $parentObject);
-                                        $this->_uow->setOriginalEntityProperty(spl_object_id($element), $inverseAssoc['fieldName'], $parentObject);
+                                if ($relation->inversedBy !== null) {
+                                    $inverseAssoc = $targetClass->associationMappings[$relation->inversedBy];
+                                    if ($inverseAssoc->isToOne()) {
+                                        $targetClass->reflFields[$inverseAssoc->fieldName]->setValue($element, $parentObject);
+                                        $this->uow->setOriginalEntityProperty(spl_object_id($element), $inverseAssoc->fieldName, $parentObject);
                                     }
                                 }
                             } else {
                                 // For sure bidirectional, as there is no inverse side in unidirectional mappings
-                                $targetClass->reflFields[$relation['mappedBy']]->setValue($element, $parentObject);
-                                $this->_uow->setOriginalEntityProperty(spl_object_id($element), $relation['mappedBy'], $parentObject);
+                                $targetClass->reflFields[$relation->mappedBy]->setValue($element, $parentObject);
+                                $this->uow->setOriginalEntityProperty(spl_object_id($element), $relation->mappedBy, $parentObject);
                             }
 
                             // Update result pointer
                             $this->resultPointers[$dqlAlias] = $element;
                         } else {
-                            $this->_uow->setOriginalEntityProperty($oid, $relationField, null);
+                            $this->uow->setOriginalEntityProperty($oid, $relationField, null);
                             $reflField->setValue($parentObject, null);
                         }
                         // else leave $reflFieldValue null for single-valued associations
@@ -504,8 +497,8 @@ class ObjectHydrator extends AbstractHydrator
                     if (isset($this->resultSetMapping()->indexByMap[$dqlAlias])) {
                         $resultKey = $row[$this->resultSetMapping()->indexByMap[$dqlAlias]];
 
-                        if (isset($this->_hints['collection'])) {
-                            $this->_hints['collection']->hydrateSet($resultKey, $element);
+                        if (isset($this->hints['collection'])) {
+                            $this->hints['collection']->hydrateSet($resultKey, $element);
                         }
 
                         $result[$resultKey] = $element;
@@ -513,8 +506,8 @@ class ObjectHydrator extends AbstractHydrator
                         $resultKey = $this->resultCounter;
                         ++$this->resultCounter;
 
-                        if (isset($this->_hints['collection'])) {
-                            $this->_hints['collection']->hydrateAdd($element);
+                        if (isset($this->hints['collection'])) {
+                            $this->hints['collection']->hydrateAdd($element);
                         }
 
                         $result[] = $element;
@@ -532,8 +525,8 @@ class ObjectHydrator extends AbstractHydrator
                 }
             }
 
-            if (isset($this->_hints[Query::HINT_INTERNAL_ITERATION]) && $this->_hints[Query::HINT_INTERNAL_ITERATION]) {
-                $this->_uow->hydrationComplete();
+            if (isset($this->hints[Query::HINT_INTERNAL_ITERATION]) && $this->hints[Query::HINT_INTERNAL_ITERATION]) {
+                $this->uow->hydrationComplete();
             }
         }
 
@@ -563,9 +556,7 @@ class ObjectHydrator extends AbstractHydrator
             $scalarCount = (isset($rowData['scalars']) ? count($rowData['scalars']) : 0);
 
             foreach ($rowData['newObjects'] as $objIndex => $newObject) {
-                $class = $newObject['class'];
-                $args  = $newObject['args'];
-                $obj   = $class->newInstanceArgs($args);
+                $obj = $newObject['obj'];
 
                 if ($scalarCount === 0 && count($rowData['newObjects']) === 1) {
                     $result[$resultKey] = $obj;
@@ -581,12 +572,8 @@ class ObjectHydrator extends AbstractHydrator
     /**
      * When executed in a hydrate() loop we may have to clear internal state to
      * decrease memory consumption.
-     *
-     * @param mixed $eventArgs
-     *
-     * @return void
      */
-    public function onClear($eventArgs)
+    public function onClear(mixed $eventArgs): void
     {
         parent::onClear($eventArgs);
 
