@@ -7,9 +7,6 @@ namespace Doctrine\ORM\Mapping;
 use Doctrine\Common\EventManager;
 use Doctrine\DBAL\Platforms;
 use Doctrine\DBAL\Platforms\AbstractPlatform;
-use Doctrine\DBAL\Platforms\MySQLPlatform;
-use Doctrine\DBAL\Platforms\SqlitePlatform;
-use Doctrine\DBAL\Platforms\SQLServerPlatform;
 use Doctrine\Deprecations\Deprecation;
 use Doctrine\ORM\EntityManagerInterface;
 use Doctrine\ORM\Event\LoadClassMetadataEventArgs;
@@ -20,8 +17,6 @@ use Doctrine\ORM\Id\AssignedGenerator;
 use Doctrine\ORM\Id\BigIntegerIdentityGenerator;
 use Doctrine\ORM\Id\IdentityGenerator;
 use Doctrine\ORM\Id\SequenceGenerator;
-use Doctrine\ORM\Id\UuidGenerator;
-use Doctrine\ORM\Mapping\Exception\CannotGenerateIds;
 use Doctrine\ORM\Mapping\Exception\InvalidCustomGenerator;
 use Doctrine\ORM\Mapping\Exception\UnknownGeneratorType;
 use Doctrine\ORM\Proxy\DefaultProxyClassNameResolver;
@@ -37,10 +32,10 @@ use function class_exists;
 use function count;
 use function end;
 use function explode;
-use function get_class;
 use function in_array;
 use function is_a;
 use function is_subclass_of;
+use function method_exists;
 use function str_contains;
 use function strlen;
 use function strtolower;
@@ -52,35 +47,22 @@ use function substr;
  * to a relational database.
  *
  * @extends AbstractClassMetadataFactory<ClassMetadata>
- * @psalm-import-type AssociationMapping from ClassMetadata
- * @psalm-import-type EmbeddedClassMapping from ClassMetadata
- * @psalm-import-type FieldMapping from ClassMetadata
  */
 class ClassMetadataFactory extends AbstractClassMetadataFactory
 {
-    /** @var EntityManagerInterface|null */
-    private $em;
-
-    /** @var AbstractPlatform|null */
-    private $targetPlatform;
-
-    /** @var MappingDriver */
-    private $driver;
-
-    /** @var EventManager */
-    private $evm;
+    private EntityManagerInterface|null $em       = null;
+    private AbstractPlatform|null $targetPlatform = null;
+    private MappingDriver|null $driver            = null;
+    private EventManager|null $evm                = null;
 
     /** @var mixed[] */
-    private $embeddablesActiveNesting = [];
+    private array $embeddablesActiveNesting = [];
 
     private const NON_IDENTITY_DEFAULT_STRATEGY = [
-        'Doctrine\DBAL\Platforms\PostgreSqlPlatform' => ClassMetadata::GENERATOR_TYPE_SEQUENCE,
         Platforms\OraclePlatform::class => ClassMetadata::GENERATOR_TYPE_SEQUENCE,
-        Platforms\PostgreSQLPlatform::class => ClassMetadata::GENERATOR_TYPE_SEQUENCE,
     ];
 
-    /** @return void */
-    public function setEntityManager(EntityManagerInterface $em)
+    public function setEntityManager(EntityManagerInterface $em): void
     {
         parent::setProxyClassNameResolver(new DefaultProxyClassNameResolver());
 
@@ -88,19 +70,47 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
     }
 
     /**
-     * {@inheritDoc}
+     * @param A $maybeOwningSide
+     *
+     * @return (A is ManyToManyAssociationMapping ? ManyToManyOwningSideMapping : (
+     *     A is OneToOneAssociationMapping ? OneToOneOwningSideMapping : (
+     *     A is OneToManyAssociationMapping ? ManyToOneAssociationMapping : (
+     *     A is ManyToOneAssociationMapping ? ManyToOneAssociationMapping :
+     *     ManyToManyOwningSideMapping|OneToOneOwningSideMapping|ManyToOneAssociationMapping
+     * ))))
+     *
+     * @template A of AssociationMapping
      */
-    protected function initialize()
+    final public function getOwningSide(AssociationMapping $maybeOwningSide): OwningSideMapping
+    {
+        if ($maybeOwningSide instanceof OwningSideMapping) {
+            assert($maybeOwningSide instanceof ManyToManyOwningSideMapping ||
+                $maybeOwningSide instanceof OneToOneOwningSideMapping ||
+                $maybeOwningSide instanceof ManyToOneAssociationMapping);
+
+            return $maybeOwningSide;
+        }
+
+        assert($maybeOwningSide instanceof InverseSideMapping);
+
+        $owningSide = $this->getMetadataFor($maybeOwningSide->targetEntity)
+            ->associationMappings[$maybeOwningSide->mappedBy];
+
+        assert($owningSide instanceof ManyToManyOwningSideMapping ||
+            $owningSide instanceof OneToOneOwningSideMapping ||
+            $owningSide instanceof ManyToOneAssociationMapping);
+
+        return $owningSide;
+    }
+
+    protected function initialize(): void
     {
         $this->driver      = $this->em->getConfiguration()->getMetadataDriverImpl();
         $this->evm         = $this->em->getEventManager();
         $this->initialized = true;
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    protected function onNotFoundMetadata($className)
+    protected function onNotFoundMetadata(string $className): ClassMetadata|null
     {
         if (! $this->evm->hasListeners(Events::onClassMetadataNotFound)) {
             return null;
@@ -118,11 +128,15 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
     /**
      * {@inheritDoc}
      */
-    protected function doLoadMetadata($class, $parent, $rootEntityFound, array $nonSuperclassParents)
-    {
+    protected function doLoadMetadata(
+        ClassMetadataInterface $class,
+        ClassMetadataInterface|null $parent,
+        bool $rootEntityFound,
+        array $nonSuperclassParents,
+    ): void {
         if ($parent) {
             $class->setInheritanceType($parent->inheritanceType);
-            $class->setDiscriminatorColumn($parent->discriminatorColumn);
+            $class->setDiscriminatorColumn($parent->discriminatorColumn === null ? null : clone $parent->discriminatorColumn);
             $class->setIdGeneratorType($parent->generatorType);
             $this->addInheritedFields($class, $parent);
             $this->addInheritedRelations($class, $parent);
@@ -162,27 +176,21 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
 
         if (! $class->isMappedSuperclass) {
             if ($rootEntityFound && $class->isInheritanceTypeNone()) {
-                Deprecation::trigger(
-                    'doctrine/orm',
-                    'https://github.com/doctrine/orm/pull/10431',
-                    "Entity class '%s' is a subclass of the root entity class '%s', but no inheritance mapping type was declared. This is a misconfiguration and will be an error in Doctrine ORM 3.0.",
-                    $class->name,
-                    end($nonSuperclassParents)
-                );
+                throw MappingException::missingInheritanceTypeDeclaration(end($nonSuperclassParents), $class->name);
             }
 
             foreach ($class->embeddedClasses as $property => $embeddableClass) {
-                if (isset($embeddableClass['inherited'])) {
+                if (isset($embeddableClass->inherited)) {
                     continue;
                 }
 
-                if (isset($this->embeddablesActiveNesting[$embeddableClass['class']])) {
+                if (isset($this->embeddablesActiveNesting[$embeddableClass->class])) {
                     throw MappingException::infiniteEmbeddableNesting($class->name, $property);
                 }
 
                 $this->embeddablesActiveNesting[$class->name] = true;
 
-                $embeddableMetadata = $this->getMetadataFor($embeddableClass['class']);
+                $embeddableMetadata = $this->getMetadataFor($embeddableClass->class);
 
                 if ($embeddableMetadata->isEmbeddedClass) {
                     $this->addNestedEmbeddedClasses($embeddableMetadata, $class, $property);
@@ -219,18 +227,6 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
                 $class->containsEnumIdentifier = true;
             }
 
-            if (! empty($parent->namedQueries)) {
-                $this->addInheritedNamedQueries($class, $parent);
-            }
-
-            if (! empty($parent->namedNativeQueries)) {
-                $this->addInheritedNamedNativeQueries($class, $parent);
-            }
-
-            if (! empty($parent->sqlResultSetMappings)) {
-                $this->addInheritedSqlResultSetMappings($class, $parent);
-            }
-
             if (! empty($parent->entityListeners) && empty($class->entityListeners)) {
                 $class->entityListeners = $parent->entityListeners;
             }
@@ -252,29 +248,15 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
 
         $this->findAbstractEntityClassesNotListedInDiscriminatorMap($class);
 
-        if ($class->changeTrackingPolicy === ClassMetadata::CHANGETRACKING_NOTIFY) {
-            Deprecation::trigger(
-                'doctrine/orm',
-                'https://github.com/doctrine/orm/issues/8383',
-                'NOTIFY Change Tracking policy used in "%s" is deprecated, use deferred explicit instead.',
-                $class->name
-            );
-        }
-
         $this->validateRuntimeMetadata($class, $parent);
     }
 
     /**
      * Validate runtime metadata is correctly defined.
      *
-     * @param ClassMetadata               $class
-     * @param ClassMetadataInterface|null $parent
-     *
-     * @return void
-     *
      * @throws MappingException
      */
-    protected function validateRuntimeMetadata($class, $parent)
+    protected function validateRuntimeMetadata(ClassMetadata $class, ClassMetadataInterface|null $parent): void
     {
         if (! $class->reflClass) {
             // only validate if there is a reflection class instance
@@ -302,7 +284,7 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
                     }
                 }
             } else {
-                assert($parent instanceof ClassMetadataInfo); // https://github.com/doctrine/orm/issues/8746
+                assert($parent instanceof ClassMetadata); // https://github.com/doctrine/orm/issues/8746
                 if (
                     ! $class->reflClass->isAbstract()
                     && ! in_array($class->name, $class->discriminatorMap, true)
@@ -316,15 +298,12 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    protected function newClassMetadataInstance($className)
+    protected function newClassMetadataInstance(string $className): ClassMetadata
     {
         return new ClassMetadata(
             $className,
             $this->em->getConfiguration()->getNamingStrategy(),
-            $this->em->getConfiguration()->getTypedFieldMapper()
+            $this->em->getConfiguration()->getTypedFieldMapper(),
         );
     }
 
@@ -412,7 +391,7 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
         $class       = $this->newClassMetadataInstance($className);
         $this->initializeReflection($class, $reflService);
 
-        $this->driver->loadMetadataForClass($className, $class);
+        $this->getDriver()->loadMetadataForClass($className, $class);
 
         return $class->isMappedSuperclass;
     }
@@ -420,7 +399,7 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
     /**
      * Gets the lower-case short name of a class.
      *
-     * @psalm-param class-string $className
+     * @param class-string $className
      */
     private function getShortName(string $className): string
     {
@@ -436,17 +415,17 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
     /**
      * Puts the `inherited` and `declared` values into mapping information for fields, associations
      * and embedded classes.
-     *
-     * @param AssociationMapping|EmbeddedClassMapping|FieldMapping $mapping
      */
-    private function addMappingInheritanceInformation(array &$mapping, ClassMetadata $parentClass): void
-    {
-        if (! isset($mapping['inherited']) && ! $parentClass->isMappedSuperclass) {
-            $mapping['inherited'] = $parentClass->name;
+    private function addMappingInheritanceInformation(
+        AssociationMapping|EmbeddedClassMapping|FieldMapping $mapping,
+        ClassMetadata $parentClass,
+    ): void {
+        if (! isset($mapping->inherited) && ! $parentClass->isMappedSuperclass) {
+            $mapping->inherited = $parentClass->name;
         }
 
-        if (! isset($mapping['declared'])) {
-            $mapping['declared'] = $parentClass->name;
+        if (! isset($mapping->declared)) {
+            $mapping->declared = $parentClass->name;
         }
     }
 
@@ -456,8 +435,9 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
     private function addInheritedFields(ClassMetadata $subClass, ClassMetadata $parentClass): void
     {
         foreach ($parentClass->fieldMappings as $mapping) {
-            $this->addMappingInheritanceInformation($mapping, $parentClass);
-            $subClass->addInheritedFieldMapping($mapping);
+            $subClassMapping = clone $mapping;
+            $this->addMappingInheritanceInformation($subClassMapping, $parentClass);
+            $subClass->addInheritedFieldMapping($subClassMapping);
         }
 
         foreach ($parentClass->reflFields as $name => $field) {
@@ -473,26 +453,28 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
     private function addInheritedRelations(ClassMetadata $subClass, ClassMetadata $parentClass): void
     {
         foreach ($parentClass->associationMappings as $field => $mapping) {
-            $this->addMappingInheritanceInformation($mapping, $parentClass);
+            $subClassMapping = clone $mapping;
+            $this->addMappingInheritanceInformation($subClassMapping, $parentClass);
             // When the class inheriting the relation ($subClass) is the first entity class since the
             // relation has been defined in a mapped superclass (or in a chain
             // of mapped superclasses) above, then declare this current entity class as the source of
             // the relationship.
             // According to the definitions given in https://github.com/doctrine/orm/pull/10396/,
             // this is the case <=> ! isset($mapping['inherited']).
-            if (! isset($mapping['inherited'])) {
-                $mapping['sourceEntity'] = $subClass->name;
+            if (! isset($subClassMapping->inherited)) {
+                $subClassMapping->sourceEntity = $subClass->name;
             }
 
-            $subClass->addInheritedAssociationMapping($mapping);
+            $subClass->addInheritedAssociationMapping($subClassMapping);
         }
     }
 
     private function addInheritedEmbeddedClasses(ClassMetadata $subClass, ClassMetadata $parentClass): void
     {
         foreach ($parentClass->embeddedClasses as $field => $embeddedClass) {
-            $this->addMappingInheritanceInformation($embeddedClass, $parentClass);
-            $subClass->embeddedClasses[$field] = $embeddedClass;
+            $subClassMapping = clone $embeddedClass;
+            $this->addMappingInheritanceInformation($subClassMapping, $parentClass);
+            $subClass->embeddedClasses[$field] = $subClassMapping;
         }
     }
 
@@ -506,25 +488,25 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
     private function addNestedEmbeddedClasses(
         ClassMetadata $subClass,
         ClassMetadata $parentClass,
-        string $prefix
+        string $prefix,
     ): void {
         foreach ($subClass->embeddedClasses as $property => $embeddableClass) {
-            if (isset($embeddableClass['inherited'])) {
+            if (isset($embeddableClass->inherited)) {
                 continue;
             }
 
-            $embeddableMetadata = $this->getMetadataFor($embeddableClass['class']);
+            $embeddableMetadata = $this->getMetadataFor($embeddableClass->class);
 
             $parentClass->mapEmbedded(
                 [
                     'fieldName' => $prefix . '.' . $property,
                     'class' => $embeddableMetadata->name,
-                    'columnPrefix' => $embeddableClass['columnPrefix'],
-                    'declaredField' => $embeddableClass['declaredField']
-                            ? $prefix . '.' . $embeddableClass['declaredField']
+                    'columnPrefix' => $embeddableClass->columnPrefix,
+                    'declaredField' => $embeddableClass->declaredField
+                            ? $prefix . '.' . $embeddableClass->declaredField
                             : $prefix,
-                    'originalField' => $embeddableClass['originalField'] ?: $property,
-                ]
+                    'originalField' => $embeddableClass->originalField ?: $property,
+                ],
             );
         }
     }
@@ -551,78 +533,13 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
         }
     }
 
-    /**
-     * Adds inherited named queries to the subclass mapping.
-     */
-    private function addInheritedNamedQueries(ClassMetadata $subClass, ClassMetadata $parentClass): void
-    {
-        foreach ($parentClass->namedQueries as $name => $query) {
-            if (! isset($subClass->namedQueries[$name])) {
-                $subClass->addNamedQuery(
-                    [
-                        'name'  => $query['name'],
-                        'query' => $query['query'],
-                    ]
-                );
-            }
-        }
-    }
-
-    /**
-     * Adds inherited named native queries to the subclass mapping.
-     */
-    private function addInheritedNamedNativeQueries(ClassMetadata $subClass, ClassMetadata $parentClass): void
-    {
-        foreach ($parentClass->namedNativeQueries as $name => $query) {
-            if (! isset($subClass->namedNativeQueries[$name])) {
-                $subClass->addNamedNativeQuery(
-                    [
-                        'name'              => $query['name'],
-                        'query'             => $query['query'],
-                        'isSelfClass'       => $query['isSelfClass'],
-                        'resultSetMapping'  => $query['resultSetMapping'],
-                        'resultClass'       => $query['isSelfClass'] ? $subClass->name : $query['resultClass'],
-                    ]
-                );
-            }
-        }
-    }
-
-    /**
-     * Adds inherited sql result set mappings to the subclass mapping.
-     */
-    private function addInheritedSqlResultSetMappings(ClassMetadata $subClass, ClassMetadata $parentClass): void
-    {
-        foreach ($parentClass->sqlResultSetMappings as $name => $mapping) {
-            if (! isset($subClass->sqlResultSetMappings[$name])) {
-                $entities = [];
-                foreach ($mapping['entities'] as $entity) {
-                    $entities[] = [
-                        'fields'                => $entity['fields'],
-                        'isSelfClass'           => $entity['isSelfClass'],
-                        'discriminatorColumn'   => $entity['discriminatorColumn'],
-                        'entityClass'           => $entity['isSelfClass'] ? $subClass->name : $entity['entityClass'],
-                    ];
-                }
-
-                $subClass->addSqlResultSetMapping(
-                    [
-                        'name'          => $mapping['name'],
-                        'columns'       => $mapping['columns'],
-                        'entities'      => $entities,
-                    ]
-                );
-            }
-        }
-    }
-
     /**
      * Completes the ID generator mapping. If "auto" is specified we choose the generator
      * most appropriate for the targeted database platform.
      *
      * @throws ORMException
      */
-    private function completeIdGeneratorMapping(ClassMetadataInfo $class): void
+    private function completeIdGeneratorMapping(ClassMetadata $class): void
     {
         $idGenType = $class->generatorType;
         if ($idGenType === ClassMetadata::GENERATOR_TYPE_AUTO) {
@@ -636,43 +553,9 @@ class ClassMetadataFactory extends AbstractClassMetadataFactory
                 $fieldName    = $class->identifier ? $class->getSingleIdentifierFieldName() : null;
                 $platform     = $this->getTargetPlatform();
 
-                // Platforms that do not have native IDENTITY support need a sequence to emulate this behaviour.
-                /** @psalm-suppress UndefinedClass, InvalidClass */
-                if (! $platform instanceof MySQLPlatform && ! $platform instanceof SqlitePlatform && ! $platform instanceof SQLServerPlatform && $platform->usesSequenceEmulatedIdentityColumns()) {
-                    Deprecation::trigger(
-                        'doctrine/orm',
-                        'https://github.com/doctrine/orm/issues/8850',
-                        <<<'DEPRECATION'
-Context: Loading metadata for class %s
-Problem: Using identity columns emulated with a sequence is deprecated and will not be possible in Doctrine ORM 3.0.
-Solution: Use the SEQUENCE generator strategy instead.
-DEPRECATION
-                            ,
-                        $class->name,
-                        get_class($this->getTargetPlatform())
-                    );
-                    $columnName     = $class->getSingleIdentifierColumnName();
-                    $quoted         = isset($class->fieldMappings[$fieldName]['quoted']) || isset($class->table['quoted']);
-                    $sequencePrefix = $class->getSequencePrefix($this->getTargetPlatform());
-                    $sequenceName   = $this->getTargetPlatform()->getIdentitySequenceName($sequencePrefix, $columnName);
-                    $definition     = [
-                        'sequenceName' => $this->truncateSequenceName($sequenceName),
-                    ];
-
-                    if ($quoted) {
-                        $definition['quoted'] = true;
-                    }
-
-                    $sequenceName = $this
-                        ->em
-                        ->getConfiguration()
-                        ->getQuoteStrategy()
-                        ->getSequenceName($definition, $class, $this->getTargetPlatform());
-                }
-
-                $generator = $fieldName && $class->fieldMappings[$fieldName]['type'] === 'bigint'
-                    ? new BigIntegerIdentityGenerator($sequenceName)
-                    : new IdentityGenerator($sequenceName);
+                $generator = $fieldName && $class->fieldMappings[$fieldName]->type === 'bigint'
+                    ? new BigIntegerIdentityGenerator()
+                    : new IdentityGenerator();
 
                 $class->setIdGenerator($generator);
 
@@ -685,7 +568,7 @@ DEPRECATION
                 if (! $definition) {
                     $fieldName    = $class->getSingleIdentifierFieldName();
                     $sequenceName = $class->getSequenceName($this->getTargetPlatform());
-                    $quoted       = isset($class->fieldMappings[$fieldName]['quoted']) || isset($class->table['quoted']);
+                    $quoted       = isset($class->fieldMappings[$fieldName]->quoted) || isset($class->table['quoted']);
 
                     $definition = [
                         'sequenceName'      => $this->truncateSequenceName($sequenceName),
@@ -702,7 +585,7 @@ DEPRECATION
 
                 $sequenceGenerator = new SequenceGenerator(
                     $this->em->getConfiguration()->getQuoteStrategy()->getSequenceName($definition, $class, $this->getTargetPlatform()),
-                    (int) $definition['allocationSize']
+                    (int) $definition['allocationSize'],
                 );
                 $class->setIdGenerator($sequenceGenerator);
                 break;
@@ -711,16 +594,6 @@ DEPRECATION
                 $class->setIdGenerator(new AssignedGenerator());
                 break;
 
-            case ClassMetadata::GENERATOR_TYPE_UUID:
-                Deprecation::trigger(
-                    'doctrine/orm',
-                    'https://github.com/doctrine/orm/issues/7312',
-                    'Mapping for %s: the "UUID" id generator strategy is deprecated with no replacement',
-                    $class->name
-                );
-                $class->setIdGenerator(new UuidGenerator());
-                break;
-
             case ClassMetadata::GENERATOR_TYPE_CUSTOM:
                 $definition = $class->customGeneratorDefinition;
                 if ($definition === null) {
@@ -739,7 +612,7 @@ DEPRECATION
         }
     }
 
-    /** @psalm-return ClassMetadata::GENERATOR_TYPE_* */
+    /** @phpstan-return ClassMetadata::GENERATOR_TYPE_* */
     private function determineIdGeneratorStrategy(AbstractPlatform $platform): int
     {
         assert($this->em !== null);
@@ -749,25 +622,36 @@ DEPRECATION
             }
         }
 
-        foreach (self::NON_IDENTITY_DEFAULT_STRATEGY as $platformFamily => $strategy) {
+        $nonIdentityDefaultStrategy = self::NON_IDENTITY_DEFAULT_STRATEGY;
+
+        // DBAL 3
+        if (method_exists($platform, 'getIdentitySequenceName')) {
+            $nonIdentityDefaultStrategy[Platforms\PostgreSQLPlatform::class] = ClassMetadata::GENERATOR_TYPE_SEQUENCE;
+        }
+
+        foreach ($nonIdentityDefaultStrategy as $platformFamily => $strategy) {
             if (is_a($platform, $platformFamily)) {
-                if ($platform instanceof Platforms\PostgreSQLPlatform || is_a($platform, 'Doctrine\DBAL\Platforms\PostgreSqlPlatform')) {
+                if ($platform instanceof Platforms\PostgreSQLPlatform) {
                     Deprecation::trigger(
                         'doctrine/orm',
                         'https://github.com/doctrine/orm/issues/8893',
                         <<<'DEPRECATION'
-Relying on non-optimal defaults for ID generation is deprecated, and IDENTITY
-results in SERIAL, which is not recommended.
-Instead, configure identifier generation strategies explicitly through
-configuration.
-We currently recommend "SEQUENCE" for "%s", so you should use
-$configuration->setIdentityGenerationPreferences([
-    "%s" => ClassMetadata::GENERATOR_TYPE_SEQUENCE,
-]);
-DEPRECATION
-                        ,
+                        Relying on non-optimal defaults for ID generation is deprecated, and IDENTITY
+                        results in SERIAL, which is not recommended.
+                        Instead, configure identifier generation strategies explicitly through
+                        configuration.
+                        We currently recommend "SEQUENCE" for "%s", when using DBAL 3,
+                        and "IDENTITY" when using DBAL 4,
+                        so you should probably use the following configuration before upgrading to DBAL 4,
+                        and remove it after deploying that upgrade:
+
+                        $configuration->setIdentityGenerationPreferences([
+                            "%s" => ClassMetadata::GENERATOR_TYPE_SEQUENCE,
+                        ]);
+
+                        DEPRECATION,
+                        $platformFamily,
                         $platformFamily,
-                        $platformFamily
                     );
                 }
 
@@ -775,21 +659,13 @@ DEPRECATION
             }
         }
 
-        if ($platform->supportsIdentityColumns()) {
-            return ClassMetadata::GENERATOR_TYPE_IDENTITY;
-        }
-
-        if ($platform->supportsSequences()) {
-            return ClassMetadata::GENERATOR_TYPE_SEQUENCE;
-        }
-
-        throw CannotGenerateIds::withPlatform($platform);
+        return ClassMetadata::GENERATOR_TYPE_IDENTITY;
     }
 
     private function truncateSequenceName(string $schemaElementName): string
     {
         $platform = $this->getTargetPlatform();
-        if (! $platform instanceof Platforms\OraclePlatform && ! $platform instanceof Platforms\SQLAnywherePlatform) {
+        if (! $platform instanceof Platforms\OraclePlatform) {
             return $schemaElementName;
         }
 
@@ -805,7 +681,7 @@ DEPRECATION
     /**
      * Inherits the ID generator mapping from a parent class.
      */
-    private function inheritIdGeneratorMapping(ClassMetadataInfo $class, ClassMetadataInfo $parent): void
+    private function inheritIdGeneratorMapping(ClassMetadata $class, ClassMetadata $parent): void
     {
         if ($parent->isIdGeneratorSequence()) {
             $class->setSequenceGeneratorDefinition($parent->sequenceGeneratorDefinition);
@@ -815,52 +691,29 @@ DEPRECATION
             $class->setIdGeneratorType($parent->generatorType);
         }
 
-        if ($parent->idGenerator) {
+        if ($parent->idGenerator ?? null) {
             $class->setIdGenerator($parent->idGenerator);
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    protected function wakeupReflection(ClassMetadataInterface $class, ReflectionService $reflService)
+    protected function wakeupReflection(ClassMetadataInterface $class, ReflectionService $reflService): void
     {
-        assert($class instanceof ClassMetadata);
         $class->wakeupReflection($reflService);
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    protected function initializeReflection(ClassMetadataInterface $class, ReflectionService $reflService)
+    protected function initializeReflection(ClassMetadataInterface $class, ReflectionService $reflService): void
     {
-        assert($class instanceof ClassMetadata);
         $class->initializeReflection($reflService);
     }
 
-    /**
-     * @deprecated This method will be removed in ORM 3.0.
-     *
-     * @return class-string
-     */
-    protected function getFqcnFromAlias($namespaceAlias, $simpleClassName)
+    protected function getDriver(): MappingDriver
     {
-        /** @psalm-var class-string */
-        return $this->em->getConfiguration()->getEntityNamespace($namespaceAlias) . '\\' . $simpleClassName;
-    }
+        assert($this->driver !== null);
 
-    /**
-     * {@inheritDoc}
-     */
-    protected function getDriver()
-    {
         return $this->driver;
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    protected function isEntity(ClassMetadataInterface $class)
+    protected function isEntity(ClassMetadataInterface $class): bool
     {
         return ! $class->isMappedSuperclass;
     }
