@@ -159,6 +159,13 @@ class ResultSetMapping
      */
     public array $newObjectMappings = [];
 
+    /**
+     * Maps object Ids in the result set to classnames.
+     *
+     * @phpstan-var array<string|int, class-string>
+     */
+    public array $newObject = [];
+
     /**
      * Maps last argument for new objects in order to initiate object construction
      *
