@@ -24,10 +24,8 @@ use function array_filter;
 use function array_keys;
 use function array_map;
 use function array_merge;
-use function array_pop;
 use function assert;
 use function count;
-use function end;
 use function implode;
 use function in_array;
 use function is_array;
@@ -84,13 +82,6 @@ class SqlWalker
      */
     private int $newObjectCounter = 0;
 
-    /**
-     * Contains nesting levels of new objects arguments
-     *
-     * @phpstan-var array<int, array{0: string|int, 1: int}>
-     */
-    private array $newObjectStack = [];
-
     private readonly EntityManagerInterface $em;
     private readonly Connection $conn;
 
@@ -1507,14 +1498,7 @@ class SqlWalker
     public function walkNewObject(AST\NewObjectExpression $newObjectExpression, string|null $newObjectResultAlias = null): string
     {
         $sqlSelectExpressions = [];
-        $objOwner             = $objOwnerIdx = null;
-
-        if ($this->newObjectStack !== []) {
-            [$objOwner, $objOwnerIdx] = end($this->newObjectStack);
-            $objIndex                 = $objOwner . ':' . $objOwnerIdx;
-        } else {
-            $objIndex = $newObjectResultAlias ?: $this->newObjectCounter++;
-        }
+        $objIndex             = $newObjectResultAlias ?: $this->newObjectCounter++;
 
         foreach ($newObjectExpression->args as $argIndex => $e) {
             $resultAlias = $this->scalarResultCounter++;
@@ -1523,10 +1507,8 @@ class SqlWalker
 
             switch (true) {
                 case $e instanceof AST\NewObjectExpression:
-                    $this->newObjectStack[] = [$objIndex, $argIndex];
-                    $sqlSelectExpressions[] = $e->dispatch($this);
-                    array_pop($this->newObjectStack);
-                    $this->rsm->nestedNewObjectArguments[$columnAlias] = ['ownerIndex' => $objIndex, 'argIndex' => $argIndex];
+                    $sqlSelectExpressions[]                            = $e->dispatch($this, $columnAlias);
+                    $this->rsm->nestedNewObjectArguments[$columnAlias] = ['ownerIndex' => $objIndex, 'argIndex' => $argIndex, 'argAlias' => $columnAlias];
                     break;
 
                 case $e instanceof AST\Subselect:
@@ -1576,12 +1558,13 @@ class SqlWalker
             $this->rsm->addScalarResult($columnAlias, $resultAlias, $fieldType);
 
             $this->rsm->newObjectMappings[$columnAlias] = [
-                'className' => $newObjectExpression->className,
                 'objIndex'  => $objIndex,
                 'argIndex'  => $argIndex,
             ];
         }
 
+        $this->rsm->newObject[$objIndex] = $newObjectExpression->className;
+
         return implode(', ', $sqlSelectExpressions);
     }
 
