@@ -8,7 +8,13 @@ use BackedEnum;
 use Doctrine\DBAL\Platforms\AbstractPlatform;
 use Doctrine\DBAL\Schema\AbstractAsset;
 use Doctrine\DBAL\Schema\AbstractSchemaManager;
+use Doctrine\DBAL\Schema\ComparatorConfig;
+use Doctrine\DBAL\Schema\ForeignKeyConstraintEditor;
 use Doctrine\DBAL\Schema\Index;
+use Doctrine\DBAL\Schema\Index\IndexedColumn;
+use Doctrine\DBAL\Schema\Name\Identifier;
+use Doctrine\DBAL\Schema\Name\UnqualifiedName;
+use Doctrine\DBAL\Schema\PrimaryKeyConstraint;
 use Doctrine\DBAL\Schema\Schema;
 use Doctrine\DBAL\Schema\Table;
 use Doctrine\ORM\EntityManagerInterface;
@@ -31,12 +37,15 @@ use function array_diff_key;
 use function array_filter;
 use function array_flip;
 use function array_intersect_key;
+use function array_map;
 use function assert;
+use function class_exists;
 use function count;
 use function current;
 use function implode;
 use function in_array;
 use function is_numeric;
+use function method_exists;
 use function strtolower;
 
 /**
@@ -282,7 +291,7 @@ class SchemaTool
                     }
 
                     if ($pkColumns !== []) {
-                        $table->setPrimaryKey($pkColumns);
+                        self::addPrimaryKeyConstraint($table, $pkColumns);
                     }
                 }
             } else {
@@ -306,7 +315,7 @@ class SchemaTool
             }
 
             if (! $table->hasIndex('primary')) {
-                $table->setPrimaryKey($pkColumns);
+                self::addPrimaryKeyConstraint($table, $pkColumns);
             }
 
             // there can be unique indexes automatically created for join column
@@ -315,7 +324,7 @@ class SchemaTool
             $primaryKey = $table->getIndex('primary');
 
             foreach ($table->getIndexes() as $idxKey => $existingIndex) {
-                if ($primaryKey->overrules($existingIndex)) {
+                if ($existingIndex !== $primaryKey && $primaryKey->spansColumns(self::getIndexedColumns($existingIndex))) {
                     $table->dropIndex($idxKey);
                 }
             }
@@ -346,7 +355,7 @@ class SchemaTool
                         }
                     }
 
-                    $table->addUniqueIndex($uniqIndex->getColumns(), is_numeric($indexName) ? null : $indexName, $indexData['options'] ?? []);
+                    $table->addUniqueIndex(self::getIndexedColumns($uniqIndex), is_numeric($indexName) ? null : $indexName, $indexData['options'] ?? []);
                 }
             }
 
@@ -572,7 +581,7 @@ class SchemaTool
                     $blacklistedFks,
                 );
 
-                $theJoinTable->setPrimaryKey($primaryKeyColumns);
+                self::addPrimaryKeyConstraint($theJoinTable, $primaryKeyColumns);
             }
         }
     }
@@ -725,7 +734,18 @@ class SchemaTool
         ) {
             foreach ($theJoinTable->getForeignKeys() as $fkName => $key) {
                 if (
-                    count(array_diff($key->getLocalColumns(), $localColumns)) === 0
+                    class_exists(ForeignKeyConstraintEditor::class)
+                    && count(array_diff(array_map(static fn (UnqualifiedName $name) => $name->toString(), $key->getReferencingColumnNames()), $localColumns)) === 0
+                    && (($key->getReferencedTableName()->toString() !== $foreignTableName)
+                    || 0 < count(array_diff(array_map(static fn (UnqualifiedName $name) => $name->toString(), $key->getReferencedColumnNames()), $foreignColumns)))
+                ) {
+                    $theJoinTable->dropForeignKey($fkName);
+                    break;
+                }
+
+                if (
+                    ! class_exists(ForeignKeyConstraintEditor::class)
+                    && count(array_diff($key->getLocalColumns(), $localColumns)) === 0
                     && (($key->getForeignTableName() !== $foreignTableName)
                     || 0 < count(array_diff($key->getForeignColumns(), $foreignColumns)))
                 ) {
@@ -843,12 +863,22 @@ class SchemaTool
             }
 
             foreach ($schema->getTables() as $table) {
-                $primaryKey = $table->getPrimaryKey();
+                if (method_exists($table, 'getPrimaryKeyConstraint')) {
+                    $primaryKey = $table->getPrimaryKeyConstraint();
+                } else {
+                    $primaryKey = $table->getPrimaryKey();
+                }
+
                 if ($primaryKey === null) {
                     continue;
                 }
 
-                $columns = $primaryKey->getColumns();
+                if ($primaryKey instanceof PrimaryKeyConstraint) {
+                    $columns = array_map(static fn (UnqualifiedName $name) => $name->toString(), $primaryKey->getColumnNames());
+                } else {
+                    $columns = self::getIndexedColumns($primaryKey);
+                }
+
                 if (count($columns) === 1) {
                     $checkSequence = $table->getName() . '_' . $columns[0] . '_seq';
                     if ($deployedSchema->hasSequence($checkSequence) && ! $schema->hasSequence($checkSequence)) {
@@ -888,7 +918,13 @@ class SchemaTool
     {
         $toSchema   = $this->getSchemaFromMetadata($classes);
         $fromSchema = $this->createSchemaForComparison($toSchema);
-        $comparator = $this->schemaManager->createComparator();
+
+        if (class_exists(ComparatorConfig::class)) {
+            $comparator = $this->schemaManager->createComparator((new ComparatorConfig())->withReportModifiedIndexes(false));
+        } else {
+            $comparator = $this->schemaManager->createComparator();
+        }
+
         $schemaDiff = $comparator->compareSchemas($fromSchema, $toSchema);
 
         return $this->platform->getAlterSchemaSQL($schemaDiff);
@@ -923,4 +959,30 @@ class SchemaTool
             $config->setSchemaAssetsFilter($previousFilter);
         }
     }
+
+    /** @param string[] $primaryKeyColumns */
+    private function addPrimaryKeyConstraint(Table $table, array $primaryKeyColumns): void
+    {
+        if (class_exists(PrimaryKeyConstraint::class)) {
+            $primaryKeyColumnNames = [];
+
+            foreach ($primaryKeyColumns as $primaryKeyColumn) {
+                $primaryKeyColumnNames[] = new UnqualifiedName(Identifier::unquoted($primaryKeyColumn));
+            }
+
+            $table->addPrimaryKeyConstraint(new PrimaryKeyConstraint(null, $primaryKeyColumnNames, true));
+        } else {
+            $table->setPrimaryKey($primaryKeyColumns);
+        }
+    }
+
+    /** @return string[] */
+    private static function getIndexedColumns(Index $index): array
+    {
+        if (method_exists(Index::class, 'getIndexedColumns')) {
+            return array_map(static fn (IndexedColumn $indexedColumn) => $indexedColumn->getColumnName()->toString(), $index->getIndexedColumns());
+        }
+
+        return $index->getColumns();
+    }
 }
