@@ -8,19 +8,14 @@ use Doctrine\ORM\Mapping\Column;
 use Doctrine\ORM\Mapping\Entity;
 use Doctrine\ORM\Mapping\Id;
 
-/** @Entity */
+#[Entity]
 class JoinedChildClass extends JoinedRootClass
 {
-    /**
-     * @var string
-     * @Column(type="string", length=255)
-     */
+    /** @var string */
+    #[Column(type: 'string', length: 255)]
     public $extension = 'ext';
 
-    /**
-     * @var string
-     * @Column(type="string", length=255)
-     * @Id
-     */
-    private $additionalId = 'additional';
+    #[Column(type: 'string', length: 255)]
+    #[Id]
+    private string $additionalId = 'additional';
 }
