@@ -8,34 +8,21 @@ use Doctrine\Common\Collections\ArrayCollection;
 use Doctrine\Common\Collections\Collection;
 use Doctrine\ORM\Mapping as ORM;
 
-/**
- * @ORM\Entity
- * @ORM\Table(name="eager_composite_join_root")
- */
+#[ORM\Entity]
+#[ORM\Table(name: 'eager_composite_join_root')]
 class RootEntity
 {
-    /**
-     * @ORM\Id
-     * @ORM\Column(type="integer", nullable=false)
-     *
-     * @var int|null
-     */
-    private $id = null;
+    #[ORM\Id]
+    #[ORM\Column(type: 'integer', nullable: false)]
+    private int|null $id = null;
 
-    /**
-     * @ORM\Id
-     * @ORM\Column(type="string", nullable=false, name="other_key")
-     *
-     * @var string
-     */
-    private $otherKey;
+    #[ORM\Id]
+    #[ORM\Column(type: 'string', nullable: false, name: 'other_key', length: 42)]
+    private string $otherKey;
 
-    /**
-     * @ORM\OneToMany(mappedBy="root", targetEntity=SecondLevel::class, fetch="EAGER")
-     *
-     * @var Collection<int, SecondLevel>
-     */
-    private $secondLevel;
+    /** @var Collection<int, SecondLevel> */
+    #[ORM\OneToMany(mappedBy: 'root', targetEntity: SecondLevel::class, fetch: 'EAGER')]
+    private Collection $secondLevel;
 
     public function __construct(int $id, string $other)
     {
@@ -44,7 +31,7 @@ class RootEntity
         $this->id          = $id;
     }
 
-    public function getId(): ?int
+    public function getId(): int|null
     {
         return $this->id;
     }
