@@ -15,7 +15,7 @@ final class AbstractFetchEagerTest extends OrmFunctionalTestCase
     {
         $this->createSchemaForModels(
             AbstractRemoteControl::class,
-            User::class
+            User::class,
         );
 
         $control = new MobileRemoteControl('smart');
