@@ -66,10 +66,11 @@ class LifecycleCallbackTest extends OrmFunctionalTestCase
         self::assertTrue($entity->postPersistCallbackInvoked);
 
         $this->_em->clear();
+        LifecycleCallbackTestEntity::$postLoadCallbackInvoked = false; // Reset the tracking of the postLoad invocation
 
         $query  = $this->_em->createQuery('select e from Doctrine\Tests\ORM\Functional\LifecycleCallbackTestEntity e');
         $result = $query->getResult();
-        self::assertTrue($result[0]->postLoadCallbackInvoked);
+        self::assertTrue($result[0]::$postLoadCallbackInvoked);
 
         $result[0]->value = 'hello again';
 
@@ -130,12 +131,14 @@ class LifecycleCallbackTest extends OrmFunctionalTestCase
         $id = $entity->getId();
 
         $this->_em->clear();
+        LifecycleCallbackTestEntity::$postLoadCallbackInvoked = false; // Reset the tracking of the postLoad invocation
 
         $reference = $this->_em->getReference(LifecycleCallbackTestEntity::class, $id);
-        self::assertFalse($reference->postLoadCallbackInvoked);
+        self::assertFalse($reference::$postLoadCallbackInvoked);
+        $this->assertTrue($this->isUninitializedObject($reference));
 
         $reference->getValue(); // trigger proxy load
-        self::assertTrue($reference->postLoadCallbackInvoked);
+        self::assertTrue($reference::$postLoadCallbackInvoked);
     }
 
     #[Group('DDC-958')]
@@ -148,13 +151,14 @@ class LifecycleCallbackTest extends OrmFunctionalTestCase
         $id = $entity->getId();
 
         $this->_em->clear();
+        LifecycleCallbackTestEntity::$postLoadCallbackInvoked = false; // Reset the tracking of the postLoad invocation
 
         $reference = $this->_em->find(LifecycleCallbackTestEntity::class, $id);
-        self::assertTrue($reference->postLoadCallbackInvoked);
-        $reference->postLoadCallbackInvoked = false;
+        self::assertTrue($reference::$postLoadCallbackInvoked);
+        $reference::$postLoadCallbackInvoked = false;
 
         $this->_em->refresh($reference);
-        self::assertTrue($reference->postLoadCallbackInvoked, 'postLoad should be invoked when refresh() is called.');
+        self::assertTrue($reference::$postLoadCallbackInvoked, 'postLoad should be invoked when refresh() is called.');
     }
 
     #[Group('DDC-113')]
@@ -195,6 +199,7 @@ class LifecycleCallbackTest extends OrmFunctionalTestCase
 
         $this->_em->flush();
         $this->_em->clear();
+        LifecycleCallbackTestEntity::$postLoadCallbackInvoked = false; // Reset the tracking of the postLoad invocation
 
         $dql = <<<'DQL'
 SELECT
@@ -212,9 +217,9 @@ DQL;
             ->createQuery(sprintf($dql, $e1->getId(), $e2->getId()))
             ->getResult();
 
-        self::assertTrue(current($entities)->postLoadCallbackInvoked);
+        self::assertTrue(current($entities)::$postLoadCallbackInvoked);
         self::assertTrue(current($entities)->postLoadCascaderNotNull);
-        self::assertTrue(current($entities)->cascader->postLoadCallbackInvoked);
+        self::assertTrue(current($entities)->cascader::$postLoadCallbackInvoked);
         self::assertEquals(current($entities)->cascader->postLoadEntitiesCount, 2);
     }
 
@@ -235,6 +240,8 @@ DQL;
 
         $this->_em->flush();
         $this->_em->clear();
+        LifecycleCallbackTestEntity::$postLoadCallbackInvoked = false; // Reset the tracking of the postLoad invocation
+        LifecycleCallbackCascader::$postLoadCallbackInvoked   = false;
 
         $dql = <<<'DQL'
 SELECT
@@ -252,7 +259,7 @@ DQL;
         $iterableResult = iterator_to_array($query->toIterable());
 
         foreach ($iterableResult as $entity) {
-            self::assertTrue($entity->postLoadCallbackInvoked);
+            self::assertTrue($entity::$postLoadCallbackInvoked);
             self::assertFalse($entity->postLoadCascaderNotNull);
 
             break;
@@ -268,15 +275,15 @@ DQL;
 
         $this->_em->flush();
         $this->_em->clear();
+        LifecycleCallbackTestEntity::$postLoadCallbackInvoked = false; // Reset the tracking of the postLoad invocation
 
-        $query = $this->_em->createQuery(
+        $query  = $this->_em->createQuery(
             'SELECT e FROM Doctrine\Tests\ORM\Functional\LifecycleCallbackTestEntity AS e',
         );
-
         $result = iterator_to_array($query->toIterable([], Query::HYDRATE_SIMPLEOBJECT));
 
         foreach ($result as $entity) {
-            self::assertTrue($entity->postLoadCallbackInvoked);
+            self::assertTrue($entity::$postLoadCallbackInvoked);
             self::assertFalse($entity->postLoadCascaderNotNull);
 
             break;
@@ -301,6 +308,8 @@ DQL;
 
         $this->_em->flush();
         $this->_em->clear();
+        LifecycleCallbackTestEntity::$postLoadCallbackInvoked = false; // Reset the tracking of the postLoad invocation
+        LifecycleCallbackCascader::$postLoadCallbackInvoked   = false;
 
         $dql = <<<'DQL'
 SELECT
@@ -318,9 +327,9 @@ DQL;
             ->createQuery($dql)->setParameter('entA_id', $entA->getId())
             ->getOneOrNullResult();
 
-        self::assertTrue($fetchedA->postLoadCallbackInvoked);
+        self::assertTrue($fetchedA::$postLoadCallbackInvoked);
         foreach ($fetchedA->entities as $fetchJoinedEntB) {
-            self::assertTrue($fetchJoinedEntB->postLoadCallbackInvoked);
+            self::assertTrue($fetchJoinedEntB::$postLoadCallbackInvoked);
         }
     }
 
@@ -455,7 +464,7 @@ class LifecycleCallbackTestEntity
     public $postPersistCallbackInvoked = false;
 
     /** @var bool */
-    public $postLoadCallbackInvoked = false;
+    public static $postLoadCallbackInvoked = false;
 
     /** @var bool */
     public $postLoadCascaderNotNull = false;
@@ -502,7 +511,7 @@ class LifecycleCallbackTestEntity
     #[PostLoad]
     public function doStuffOnPostLoad(): void
     {
-        $this->postLoadCallbackInvoked = true;
+        self::$postLoadCallbackInvoked = true;
         $this->postLoadCascaderNotNull = isset($this->cascader);
     }
 
@@ -526,7 +535,7 @@ class LifecycleCallbackCascader
 {
     /* test stuff */
     /** @var bool */
-    public $postLoadCallbackInvoked = false;
+    public static $postLoadCallbackInvoked = false;
 
     /** @var int */
     public $postLoadEntitiesCount = 0;
@@ -548,7 +557,7 @@ class LifecycleCallbackCascader
     #[PostLoad]
     public function doStuffOnPostLoad(): void
     {
-        $this->postLoadCallbackInvoked = true;
+        self::$postLoadCallbackInvoked = true;
         $this->postLoadEntitiesCount   = count($this->entities);
     }
 
