@@ -11,6 +11,7 @@ use Doctrine\ORM\Query\QueryException;
 use Doctrine\Tests\Models\CMS\CmsAddress;
 use Doctrine\Tests\Models\CMS\CmsAddressDTO;
 use Doctrine\Tests\Models\CMS\CmsAddressDTONamedArgs;
+use Doctrine\Tests\Models\CMS\CmsDumbDTO;
 use Doctrine\Tests\Models\CMS\CmsEmail;
 use Doctrine\Tests\Models\CMS\CmsPhonenumber;
 use Doctrine\Tests\Models\CMS\CmsUser;
@@ -1031,7 +1032,8 @@ class NewOperatorTest extends OrmFunctionalTestCase
                         a.country,
                         a.city,
                         a.zip,
-                        new CmsAddressDTO(
+                        \'Abbey Road\',
+                        new CmsDumbDTO(
                             a.country,
                             a.city,
                             a.zip
@@ -1078,6 +1080,18 @@ class NewOperatorTest extends OrmFunctionalTestCase
         self::assertSame($this->fixtures[1]->address->country, $result[1]['user']->address->country);
         self::assertSame($this->fixtures[2]->address->country, $result[2]['user']->address->country);
 
+        self::assertInstanceOf(CmsDumbDTO::class, $result[0]['user']->address->other);
+        self::assertInstanceOf(CmsDumbDTO::class, $result[1]['user']->address->other);
+        self::assertInstanceOf(CmsDumbDTO::class, $result[2]['user']->address->other);
+
+        self::assertSame($this->fixtures[0]->address->country, $result[0]['user']->address->other->val1);
+        self::assertSame($this->fixtures[1]->address->country, $result[1]['user']->address->other->val1);
+        self::assertSame($this->fixtures[2]->address->country, $result[2]['user']->address->other->val1);
+
+        self::assertSame($this->fixtures[0]->address->city, $result[0]['user']->address->other->val2);
+        self::assertSame($this->fixtures[1]->address->city, $result[1]['user']->address->other->val2);
+        self::assertSame($this->fixtures[2]->address->city, $result[2]['user']->address->other->val2);
+
         self::assertSame($this->fixtures[0]->status, $result[0]['status']);
         self::assertSame($this->fixtures[1]->status, $result[1]['status']);
         self::assertSame($this->fixtures[2]->status, $result[2]['status']);
@@ -1087,6 +1101,135 @@ class NewOperatorTest extends OrmFunctionalTestCase
         self::assertSame($this->fixtures[2]->username, $result[2]['cmsUserUsername']);
     }
 
+    public function testShouldSupportNestedNewOperatorsWithDtoFirst(): void
+    {
+        $dql = '
+            SELECT
+                new CmsUserDTO(
+                    u.name,
+                    e.email,
+                    new CmsAddressDTO(
+                        a.country,
+                        a.city,
+                        a.zip
+                    ),
+                    555812452
+                ) as user,
+                u.status,
+                u.username as cmsUserUsername
+            FROM
+                Doctrine\Tests\Models\CMS\CmsUser u
+            JOIN
+                u.email e
+            JOIN
+                u.address a
+            ORDER BY
+                u.name';
+
+        $query  = $this->getEntityManager()->createQuery($dql);
+        $result = $query->getResult();
+
+        self::assertCount(3, $result);
+
+        self::assertInstanceOf(CmsUserDTO::class, $result[0]['user']);
+        self::assertInstanceOf(CmsUserDTO::class, $result[1]['user']);
+        self::assertInstanceOf(CmsUserDTO::class, $result[2]['user']);
+
+        self::assertInstanceOf(CmsAddressDTO::class, $result[0]['user']->address);
+        self::assertInstanceOf(CmsAddressDTO::class, $result[1]['user']->address);
+        self::assertInstanceOf(CmsAddressDTO::class, $result[2]['user']->address);
+
+        self::assertSame($this->fixtures[0]->name, $result[0]['user']->name);
+        self::assertSame($this->fixtures[1]->name, $result[1]['user']->name);
+        self::assertSame($this->fixtures[2]->name, $result[2]['user']->name);
+
+        self::assertSame($this->fixtures[0]->email->email, $result[0]['user']->email);
+        self::assertSame($this->fixtures[1]->email->email, $result[1]['user']->email);
+        self::assertSame($this->fixtures[2]->email->email, $result[2]['user']->email);
+
+        self::assertSame($this->fixtures[0]->address->city, $result[0]['user']->address->city);
+        self::assertSame($this->fixtures[1]->address->city, $result[1]['user']->address->city);
+        self::assertSame($this->fixtures[2]->address->city, $result[2]['user']->address->city);
+
+        self::assertSame($this->fixtures[0]->address->country, $result[0]['user']->address->country);
+        self::assertSame($this->fixtures[1]->address->country, $result[1]['user']->address->country);
+        self::assertSame($this->fixtures[2]->address->country, $result[2]['user']->address->country);
+
+        self::assertSame(555812452, $result[0]['user']->phonenumbers);
+        self::assertSame(555812452, $result[1]['user']->phonenumbers);
+        self::assertSame(555812452, $result[2]['user']->phonenumbers);
+
+        self::assertSame($this->fixtures[0]->status, $result[0]['status']);
+        self::assertSame($this->fixtures[1]->status, $result[1]['status']);
+        self::assertSame($this->fixtures[2]->status, $result[2]['status']);
+
+        self::assertSame($this->fixtures[0]->username, $result[0]['cmsUserUsername']);
+        self::assertSame($this->fixtures[1]->username, $result[1]['cmsUserUsername']);
+        self::assertSame($this->fixtures[2]->username, $result[2]['cmsUserUsername']);
+    }
+
+    public function testOnlyObjectInObject(): void
+    {
+        $dql = '
+            SELECT
+                new CmsDumbDTO(
+                    new CmsDumbDTO(
+                        u.name,
+                        e.email
+                    ),
+                    new CmsAddressDTO(
+                        a.country,
+                        a.city,
+                        a.zip
+                    )
+                ) as user
+            FROM
+                Doctrine\Tests\Models\CMS\CmsUser u
+            JOIN
+                u.email e
+            JOIN
+                u.address a
+            ORDER BY
+                u.name';
+
+        $query  = $this->getEntityManager()->createQuery($dql);
+        $result = $query->getResult();
+
+        self::assertCount(3, $result);
+
+        self::assertInstanceOf(CmsDumbDTO::class, $result[0]);
+        self::assertInstanceOf(CmsDumbDTO::class, $result[1]);
+        self::assertInstanceOf(CmsDumbDTO::class, $result[2]);
+
+        self::assertInstanceOf(CmsDumbDTO::class, $result[0]->val1);
+        self::assertInstanceOf(CmsDumbDTO::class, $result[1]->val1);
+        self::assertInstanceOf(CmsDumbDTO::class, $result[2]->val1);
+
+        self::assertSame($this->fixtures[0]->name, $result[0]->val1->val1);
+        self::assertSame($this->fixtures[1]->name, $result[1]->val1->val1);
+        self::assertSame($this->fixtures[2]->name, $result[2]->val1->val1);
+
+        self::assertSame($this->fixtures[0]->email->email, $result[0]->val1->val2);
+        self::assertSame($this->fixtures[1]->email->email, $result[1]->val1->val2);
+        self::assertSame($this->fixtures[2]->email->email, $result[2]->val1->val2);
+
+        self::assertInstanceOf(CmsAddressDTO::class, $result[0]->val2);
+        self::assertInstanceOf(CmsAddressDTO::class, $result[1]->val2);
+        self::assertInstanceOf(CmsAddressDTO::class, $result[2]->val2);
+
+        self::assertSame($this->fixtures[0]->address->country, $result[0]->val2->country);
+        self::assertSame($this->fixtures[1]->address->country, $result[1]->val2->country);
+        self::assertSame($this->fixtures[2]->address->country, $result[2]->val2->country);
+
+        self::assertSame($this->fixtures[0]->address->city, $result[0]->val2->city);
+        self::assertSame($this->fixtures[1]->address->city, $result[1]->val2->city);
+        self::assertSame($this->fixtures[2]->address->city, $result[2]->val2->city);
+
+        self::assertSame($this->fixtures[0]->address->zip, $result[0]->val2->zip);
+        self::assertSame($this->fixtures[1]->address->zip, $result[1]->val2->zip);
+        self::assertSame($this->fixtures[2]->address->zip, $result[2]->val2->zip);
+    }
+
     public function testNamedArguments(): void
     {
         $dql = <<<'SQL'
@@ -1159,6 +1302,111 @@ class NewOperatorTest extends OrmFunctionalTestCase
         self::assertSame($this->fixtures[2]->username, $result[2]['cmsUserUsername']);
     }
 
+    public function testShouldSupportNestedNewOperatorsWithNestedDtoNotLast(): void
+    {
+        $dql = '
+            SELECT
+                new CmsUserDTO(
+                    u.name,
+                    e.email,
+                    new CmsAddressDTO(
+                        a.country,
+                        a.city,
+                        a.zip,
+                        \'Abbey Road\',
+                        new CmsDumbDTO(
+                            a.country,
+                            a.city,
+                            new CmsDumbDTO(
+                                a.zip,
+                                456
+                            ),
+                            a.zip
+                        )
+                    ),
+                    555812452
+                ) as user,
+                u.status,
+                u.username as cmsUserUsername
+            FROM
+                Doctrine\Tests\Models\CMS\CmsUser u
+            JOIN
+                u.email e
+            JOIN
+                u.address a
+            ORDER BY
+                u.name';
+
+        $query  = $this->getEntityManager()->createQuery($dql);
+        $result = $query->getResult();
+
+        self::assertCount(3, $result);
+
+        self::assertInstanceOf(CmsUserDTO::class, $result[0]['user']);
+        self::assertInstanceOf(CmsUserDTO::class, $result[1]['user']);
+        self::assertInstanceOf(CmsUserDTO::class, $result[2]['user']);
+
+        self::assertSame($this->fixtures[0]->name, $result[0]['user']->name);
+        self::assertSame($this->fixtures[1]->name, $result[1]['user']->name);
+        self::assertSame($this->fixtures[2]->name, $result[2]['user']->name);
+
+        self::assertSame($this->fixtures[0]->email->email, $result[0]['user']->email);
+        self::assertSame($this->fixtures[1]->email->email, $result[1]['user']->email);
+        self::assertSame($this->fixtures[2]->email->email, $result[2]['user']->email);
+
+        self::assertInstanceOf(CmsAddressDTO::class, $result[0]['user']->address);
+        self::assertInstanceOf(CmsAddressDTO::class, $result[1]['user']->address);
+        self::assertInstanceOf(CmsAddressDTO::class, $result[2]['user']->address);
+
+        self::assertSame($this->fixtures[0]->address->country, $result[0]['user']->address->country);
+        self::assertSame($this->fixtures[1]->address->country, $result[1]['user']->address->country);
+        self::assertSame($this->fixtures[2]->address->country, $result[2]['user']->address->country);
+
+        self::assertSame($this->fixtures[2]->address->city, $result[2]['user']->address->city);
+        self::assertSame($this->fixtures[0]->address->city, $result[0]['user']->address->city);
+        self::assertSame($this->fixtures[1]->address->city, $result[1]['user']->address->city);
+
+        self::assertInstanceOf(CmsDumbDTO::class, $result[0]['user']->address->other);
+        self::assertInstanceOf(CmsDumbDTO::class, $result[1]['user']->address->other);
+        self::assertInstanceOf(CmsDumbDTO::class, $result[2]['user']->address->other);
+
+        self::assertSame($this->fixtures[0]->address->country, $result[0]['user']->address->other->val1);
+        self::assertSame($this->fixtures[1]->address->country, $result[1]['user']->address->other->val1);
+        self::assertSame($this->fixtures[2]->address->country, $result[2]['user']->address->other->val1);
+
+        self::assertSame($this->fixtures[0]->address->city, $result[0]['user']->address->other->val2);
+        self::assertSame($this->fixtures[1]->address->city, $result[1]['user']->address->other->val2);
+        self::assertSame($this->fixtures[2]->address->city, $result[2]['user']->address->other->val2);
+
+        self::assertInstanceOf(CmsDumbDTO::class, $result[0]['user']->address->other->val3);
+        self::assertInstanceOf(CmsDumbDTO::class, $result[1]['user']->address->other->val3);
+        self::assertInstanceOf(CmsDumbDTO::class, $result[2]['user']->address->other->val3);
+
+        self::assertSame($this->fixtures[0]->address->zip, $result[0]['user']->address->other->val3->val1);
+        self::assertSame($this->fixtures[1]->address->zip, $result[1]['user']->address->other->val3->val1);
+        self::assertSame($this->fixtures[2]->address->zip, $result[2]['user']->address->other->val3->val1);
+
+        self::assertSame(456, $result[0]['user']->address->other->val3->val2);
+        self::assertSame(456, $result[1]['user']->address->other->val3->val2);
+        self::assertSame(456, $result[2]['user']->address->other->val3->val2);
+
+        self::assertSame($this->fixtures[0]->address->zip, $result[0]['user']->address->other->val4);
+        self::assertSame($this->fixtures[1]->address->zip, $result[1]['user']->address->other->val4);
+        self::assertSame($this->fixtures[2]->address->zip, $result[2]['user']->address->other->val4);
+
+        self::assertSame(555812452, $result[0]['user']->phonenumbers);
+        self::assertSame(555812452, $result[1]['user']->phonenumbers);
+        self::assertSame(555812452, $result[2]['user']->phonenumbers);
+
+        self::assertSame($this->fixtures[0]->status, $result[0]['status']);
+        self::assertSame($this->fixtures[1]->status, $result[1]['status']);
+        self::assertSame($this->fixtures[2]->status, $result[2]['status']);
+
+        self::assertSame($this->fixtures[0]->username, $result[0]['cmsUserUsername']);
+        self::assertSame($this->fixtures[1]->username, $result[1]['cmsUserUsername']);
+        self::assertSame($this->fixtures[2]->username, $result[2]['cmsUserUsername']);
+    }
+
     public function testVariadicArgument(): void
     {
         $dql = <<<'SQL'
