@@ -21,12 +21,14 @@ use Doctrine\ORM\Mapping\ReflectionEmbeddedProperty;
 use Doctrine\ORM\Query\QueryException;
 use Doctrine\Persistence\Reflection\RuntimeReflectionProperty;
 use Doctrine\Tests\OrmFunctionalTestCase;
+use PHPUnit\Framework\Attributes\DataProvider;
+use PHPUnit\Framework\Attributes\Group;
 use ReflectionProperty;
 
 use function class_exists;
 use function sprintf;
 
-/** @group DDC-93 */
+#[Group('DDC-93')]
 class ValueObjectsTest extends OrmFunctionalTestCase
 {
     protected function setUp(): void
@@ -39,7 +41,7 @@ class ValueObjectsTest extends OrmFunctionalTestCase
             DDC93Vehicle::class,
             DDC93Car::class,
             DDC3027Animal::class,
-            DDC3027Dog::class
+            DDC3027Dog::class,
         );
     }
 
@@ -50,17 +52,17 @@ class ValueObjectsTest extends OrmFunctionalTestCase
         if (class_exists(CommonRuntimePublicReflectionProperty::class)) {
             self::assertInstanceOf(
                 CommonRuntimePublicReflectionProperty::class,
-                $classMetadata->getReflectionProperty('address')
+                $classMetadata->getReflectionProperty('address'),
             );
         } elseif (class_exists(RuntimeReflectionProperty::class)) {
             self::assertInstanceOf(
                 RuntimeReflectionProperty::class,
-                $classMetadata->getReflectionProperty('address')
+                $classMetadata->getReflectionProperty('address'),
             );
         } else {
             self::assertInstanceOf(
                 ReflectionProperty::class,
-                $classMetadata->getReflectionProperty('address')
+                $classMetadata->getReflectionProperty('address'),
             );
         }
 
@@ -159,7 +161,7 @@ class ValueObjectsTest extends OrmFunctionalTestCase
         }
     }
 
-    /** @group dql */
+    #[Group('dql')]
     public function testDqlOnEmbeddedObjectsField(): void
     {
         if ($this->isSecondLevelCacheEnabled) {
@@ -182,7 +184,7 @@ class ValueObjectsTest extends OrmFunctionalTestCase
             $this->_em->createQuery($selectDql)
                 ->setParameter('city', 'asdf')
                 ->setParameter('country', 'Germany')
-                ->getOneOrNullResult()
+                ->getOneOrNullResult(),
         );
 
         // UPDATE
@@ -327,18 +329,18 @@ class ValueObjectsTest extends OrmFunctionalTestCase
         self::assertTrue($isFieldMapped);
     }
 
-    /** @dataProvider getInfiniteEmbeddableNestingData */
+    #[DataProvider('getInfiniteEmbeddableNestingData')]
     public function testThrowsExceptionOnInfiniteEmbeddableNesting(
         string $embeddableClassName,
-        string $declaredEmbeddableClassName
+        string $declaredEmbeddableClassName,
     ): void {
         $this->expectException(MappingException::class);
         $this->expectExceptionMessage(
             sprintf(
                 'Infinite nesting detected for embedded property %s::nested. ' .
                 'You cannot embed an embeddable from the same type inside an embeddable.',
-                __NAMESPACE__ . '\\' . $declaredEmbeddableClassName
-            )
+                __NAMESPACE__ . '\\' . $declaredEmbeddableClassName,
+            ),
         );
 
         $this->createSchemaForModels(__NAMESPACE__ . '\\' . $embeddableClassName);
@@ -355,406 +357,267 @@ class ValueObjectsTest extends OrmFunctionalTestCase
 }
 
 
-/** @Entity */
+#[Entity]
 class DDC93Person
 {
-    /**
-     * @var int
-     * @Id
-     * @GeneratedValue
-     * @Column(type="integer")
-     */
+    /** @var int */
+    #[Id]
+    #[GeneratedValue]
+    #[Column(type: 'integer')]
     public $id;
 
-    /**
-     * @var string|null
-     * @Column(type="string", length=255)
-     */
-    public $name;
-
-    /**
-     * @var DDC93Address|null
-     * @Embedded(class="DDC93Address")
-     */
-    public $address;
-
-    /**
-     * @var DDC93Timestamps
-     * @Embedded(class = "DDC93Timestamps")
-     */
+    /** @var DDC93Timestamps */
+    #[Embedded(class: 'DDC93Timestamps')]
     public $timestamps;
 
-    public function __construct(?string $name = null, ?DDC93Address $address = null)
-    {
-        $this->name       = $name;
-        $this->address    = $address;
+    public function __construct(
+        /** @var string|null */
+        #[Column(type: 'string', length: 255)]
+        public $name = null,
+        #[Embedded(class: 'DDC93Address')]
+        public DDC93Address|null $address = null,
+    ) {
         $this->timestamps = new DDC93Timestamps(new DateTime());
     }
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDC93Timestamps
 {
-    /**
-     * @var DateTime
-     * @Column(type = "datetime")
-     */
-    public $createdAt;
-
-    public function __construct(DateTime $createdAt)
-    {
-        $this->createdAt = $createdAt;
+    public function __construct(
+        #[Column(type: 'datetime')]
+        public DateTime $createdAt,
+    ) {
     }
 }
 
-/**
- * @Entity
- * @InheritanceType("SINGLE_TABLE")
- * @DiscriminatorColumn(name = "t", type = "string", length = 10)
- * @DiscriminatorMap({
- *     "v" = "Doctrine\Tests\ORM\Functional\DDC93Car",
- * })
- */
+#[Entity]
+#[InheritanceType('SINGLE_TABLE')]
+#[DiscriminatorColumn(name: 't', type: 'string', length: 10)]
+#[DiscriminatorMap(['v' => 'Doctrine\Tests\ORM\Functional\DDC93Car'])]
 abstract class DDC93Vehicle
 {
-    /**
-     * @var int
-     * @Id
-     * @GeneratedValue(strategy = "AUTO")
-     * @Column(type = "integer")
-     */
+    /** @var int */
+    #[Id]
+    #[GeneratedValue(strategy: 'AUTO')]
+    #[Column(type: 'integer')]
     public $id;
 
-    /**
-     * @var DDC93Address
-     * @Embedded(class = "DDC93Address")
-     */
-    public $address;
-
-    public function __construct(DDC93Address $address)
-    {
-        $this->address = $address;
+    public function __construct(
+        #[Embedded(class: 'DDC93Address')]
+        public DDC93Address $address,
+    ) {
     }
 }
 
-/** @Entity */
+#[Entity]
 class DDC93Car extends DDC93Vehicle
 {
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDC93Country
 {
-    /**
-     * @var string|null
-     * @Column(type="string", nullable=true)
-     */
-    public $name;
-
-    public function __construct(?string $name = null)
-    {
-        $this->name = $name;
+    public function __construct(
+        #[Column(type: 'string', nullable: true)]
+        public string|null $name = null,
+    ) {
     }
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDC93Address
 {
-    /**
-     * @var string|null
-     * @Column(type="string", length=255)
-     */
-    public $street;
+    #[Embedded(class: DDC93Country::class)]
+    public DDC93Country|null $country = null;
 
     /**
-     * @var string|null
-     * @Column(type="string", length=255)
+     * @param string|null $street
+     * @param string|null $zip
      */
-    public $zip;
-
-    /**
-     * @var string|null
-     * @Column(type="string", length=255)
-     */
-    public $city;
-
-    /**
-     * @var DDC93Country|null
-     * @Embedded(class = "DDC93Country")
-     */
-    public $country;
-
     public function __construct(
-        ?string $street = null,
-        ?string $zip = null,
-        ?string $city = null,
-        ?DDC93Country $country = null
+        #[Column(type: 'string', length: 255)]
+        public $street = null,
+        #[Column(type: 'string', length: 255)]
+        public $zip = null,
+        #[Column(type: 'string', length: 255)]
+        public string|null $city = null,
+        DDC93Country|null $country = null,
     ) {
-        $this->street  = $street;
-        $this->zip     = $zip;
-        $this->city    = $city;
         $this->country = $country;
     }
 }
 
-/** @Entity */
+#[Entity]
 class DDC93Customer
 {
-    /**
-     * @var int
-     * @Id
-     * @GeneratedValue
-     * @Column(type="integer")
-     */
-    private $id;
+    #[Id]
+    #[GeneratedValue]
+    #[Column(type: 'integer')]
+    private int $id;
 
-    /**
-     * @var DDC93ContactInfo
-     * @Embedded(class = "DDC93ContactInfo", columnPrefix = "contact_info_")
-     */
-    private $contactInfo;
+    #[Embedded(class: 'DDC93ContactInfo', columnPrefix: 'contact_info_')]
+    private DDC93ContactInfo $contactInfo;
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDC93ContactInfo
 {
-    /**
-     * @var string
-     * @Column(type="string", length=255)
-     */
+    /** @var string */
+    #[Column(type: 'string', length: 255)]
     public $email;
 
-    /**
-     * @var DDC93Address
-     * @Embedded(class = "DDC93Address")
-     */
+    /** @var DDC93Address */
+    #[Embedded(class: 'DDC93Address')]
     public $address;
 }
 
-/** @Entity */
+#[Entity]
 class DDC3028PersonWithPrefix
 {
-    /**
-     * @var DDC3028Id|null
-     * @Embedded(class="DDC3028Id", columnPrefix = "foobar_")
-     */
-    public $id;
-
-    /**
-     * @var DDC3028NestedEmbeddable|null
-     * @Embedded(class="DDC3028NestedEmbeddable", columnPrefix = "bloo_")
-     */
-    public $nested;
-
-    public function __construct(?DDC3028Id $id = null, ?DDC3028NestedEmbeddable $nested = null)
-    {
-        $this->id     = $id;
-        $this->nested = $nested;
+    public function __construct(
+        #[Embedded(class: 'DDC3028Id', columnPrefix: 'foobar_')]
+        public DDC3028Id|null $id = null,
+        #[Embedded(class: 'DDC3028NestedEmbeddable', columnPrefix: 'bloo_')]
+        public DDC3028NestedEmbeddable|null $nested = null,
+    ) {
     }
 }
 
-/** @Entity */
+#[Entity]
 class DDC3028PersonEmptyPrefix
 {
-    /**
-     * @var DDC3028Id|null
-     * @Embedded(class="DDC3028Id", columnPrefix = "")
-     */
-    public $id;
-
-    /**
-     * @var DDC3028NestedEmbeddable|null
-     * @Embedded(class="DDC3028NestedEmbeddable", columnPrefix = "")
-     */
-    public $nested;
-
-    public function __construct(?DDC3028Id $id = null, ?DDC3028NestedEmbeddable $nested = null)
-    {
-        $this->id     = $id;
-        $this->nested = $nested;
+    public function __construct(
+        #[Embedded(class: 'DDC3028Id', columnPrefix: '')]
+        public DDC3028Id|null $id = null,
+        #[Embedded(class: 'DDC3028NestedEmbeddable', columnPrefix: '')]
+        public DDC3028NestedEmbeddable|null $nested = null,
+    ) {
     }
 }
 
-/** @Entity */
+#[Entity]
 class DDC3028PersonPrefixFalse
 {
-    /**
-     * @var DDC3028Id|null
-     * @Embedded(class="DDC3028Id", columnPrefix = false)
-     */
-    public $id;
-
-    public function __construct(?DDC3028Id $id = null)
-    {
-        $this->id = $id;
+    public function __construct(
+        #[Embedded(class: 'DDC3028Id', columnPrefix: false)]
+        public DDC3028Id|null $id = null,
+    ) {
     }
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDC3028Id
 {
-    /**
-     * @var string|null
-     * @Id
-     * @Column(type="string", length=255)
-     */
-    public $id;
-
-    public function __construct(?string $id = null)
-    {
-        $this->id = $id;
+    public function __construct(
+        #[Id]
+        #[Column(type: 'string', length: 255)]
+        public string|null $id = null,
+    ) {
     }
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDC3028NestedEmbeddable
 {
-    /**
-     * @var DDC3028Id|null
-     * @Embedded(class="DDC3028Id", columnPrefix = "foo_")
-     */
-    public $nestedWithPrefix;
-
-    /**
-     * @var DDC3028Id|null
-     * @Embedded(class="DDC3028Id", columnPrefix = "")
-     */
-    public $nestedWithEmptyPrefix;
-
-    /**
-     * @var DDC3028Id|null
-     * @Embedded(class="DDC3028Id", columnPrefix = false)
-     */
-    public $nestedWithPrefixFalse;
-
     public function __construct(
-        ?DDC3028Id $nestedWithPrefix = null,
-        ?DDC3028Id $nestedWithEmptyPrefix = null,
-        ?DDC3028Id $nestedWithPrefixFalse = null
+        #[Embedded(class: 'DDC3028Id', columnPrefix: 'foo_')]
+        public DDC3028Id|null $nestedWithPrefix = null,
+        #[Embedded(class: 'DDC3028Id', columnPrefix: '')]
+        public DDC3028Id|null $nestedWithEmptyPrefix = null,
+        #[Embedded(class: 'DDC3028Id', columnPrefix: false)]
+        public DDC3028Id|null $nestedWithPrefixFalse = null,
     ) {
-        $this->nestedWithPrefix      = $nestedWithPrefix;
-        $this->nestedWithEmptyPrefix = $nestedWithEmptyPrefix;
-        $this->nestedWithPrefixFalse = $nestedWithPrefixFalse;
     }
 }
 
-/** @MappedSuperclass */
+#[MappedSuperclass]
 abstract class DDC3027Animal
 {
-    /**
-     * @var int
-     * @Id
-     * @GeneratedValue(strategy = "AUTO")
-     * @Column(type = "integer")
-     */
+    /** @var int */
+    #[Id]
+    #[GeneratedValue(strategy: 'AUTO')]
+    #[Column(type: 'integer')]
     public $id;
 
-    /**
-     * @var DDC93Address
-     * @Embedded(class = "DDC93Address")
-     */
+    /** @var DDC93Address */
+    #[Embedded(class: 'DDC93Address')]
     public $address;
 }
 
-/** @Entity */
+#[Entity]
 class DDC3027Dog extends DDC3027Animal
 {
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDCInfiniteNestingEmbeddable
 {
-    /**
-     * @var DDCInfiniteNestingEmbeddable
-     * @Embedded(class="DDCInfiniteNestingEmbeddable")
-     */
+    /** @var DDCInfiniteNestingEmbeddable */
+    #[Embedded(class: 'DDCInfiniteNestingEmbeddable')]
     public $nested;
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDCNestingEmbeddable1
 {
-    /**
-     * @var DDC3028Id
-     * @Embedded(class="DDC3028Id")
-     */
+    /** @var DDC3028Id */
+    #[Embedded(class: 'DDC3028Id')]
     public $id1;
 
-    /**
-     * @var DDC3028Id
-     * @Embedded(class="DDC3028Id")
-     */
+    /** @var DDC3028Id */
+    #[Embedded(class: 'DDC3028Id')]
     public $id2;
 
-    /**
-     * @var DDCNestingEmbeddable2
-     * @Embedded(class="DDCNestingEmbeddable2")
-     */
+    /** @var DDCNestingEmbeddable2 */
+    #[Embedded(class: 'DDCNestingEmbeddable2')]
     public $nested;
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDCNestingEmbeddable2
 {
-    /**
-     * @var DDC3028Id
-     * @Embedded(class="DDC3028Id")
-     */
+    /** @var DDC3028Id */
+    #[Embedded(class: 'DDC3028Id')]
     public $id1;
 
-    /**
-     * @var DDC3028Id
-     * @Embedded(class="DDC3028Id")
-     */
+    /** @var DDC3028Id */
+    #[Embedded(class: 'DDC3028Id')]
     public $id2;
 
-    /**
-     * @var DDCNestingEmbeddable3
-     * @Embedded(class="DDCNestingEmbeddable3")
-     */
+    /** @var DDCNestingEmbeddable3 */
+    #[Embedded(class: 'DDCNestingEmbeddable3')]
     public $nested;
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDCNestingEmbeddable3
 {
-    /**
-     * @var DDC3028Id
-     * @Embedded(class="DDC3028Id")
-     */
+    /** @var DDC3028Id */
+    #[Embedded(class: 'DDC3028Id')]
     public $id1;
 
-    /**
-     * @var DDC3028Id
-     * @Embedded(class="DDC3028Id")
-     */
+    /** @var DDC3028Id */
+    #[Embedded(class: 'DDC3028Id')]
     public $id2;
 
-    /**
-     * @var DDCNestingEmbeddable4
-     * @Embedded(class="DDCNestingEmbeddable4")
-     */
+    /** @var DDCNestingEmbeddable4 */
+    #[Embedded(class: 'DDCNestingEmbeddable4')]
     public $nested;
 }
 
-/** @Embeddable */
+#[Embeddable]
 class DDCNestingEmbeddable4
 {
-    /**
-     * @var DDC3028Id
-     * @Embedded(class="DDC3028Id")
-     */
+    /** @var DDC3028Id */
+    #[Embedded(class: 'DDC3028Id')]
     public $id1;
 
-    /**
-     * @var DDC3028Id
-     * @Embedded(class="DDC3028Id")
-     */
+    /** @var DDC3028Id */
+    #[Embedded(class: 'DDC3028Id')]
     public $id2;
 
-    /**
-     * @var DDCNestingEmbeddable1
-     * @Embedded(class="DDCNestingEmbeddable1")
-     */
+    /** @var DDCNestingEmbeddable1 */
+    #[Embedded(class: 'DDCNestingEmbeddable1')]
     public $nested;
 }
