@@ -8,20 +8,26 @@ use Doctrine\ORM\Mapping\Builder\ClassMetadataBuilder;
 use Doctrine\ORM\Mapping\Builder\EmbeddedBuilder;
 use Doctrine\ORM\Mapping\Builder\FieldBuilder;
 use Doctrine\ORM\Mapping\ClassMetadata;
+use Doctrine\ORM\Mapping\DiscriminatorColumnMapping;
+use Doctrine\ORM\Mapping\EmbeddedClassMapping;
+use Doctrine\ORM\Mapping\FieldMapping;
+use Doctrine\ORM\Mapping\ManyToManyOwningSideMapping;
+use Doctrine\ORM\Mapping\ManyToOneAssociationMapping;
 use Doctrine\ORM\Mapping\MappingException;
+use Doctrine\ORM\Mapping\OneToManyAssociationMapping;
+use Doctrine\ORM\Mapping\OneToOneOwningSideMapping;
 use Doctrine\Persistence\Mapping\RuntimeReflectionService;
 use Doctrine\Tests\Models\CMS\CmsGroup;
 use Doctrine\Tests\Models\CMS\CmsUser;
 use Doctrine\Tests\Models\ValueObjects\Name;
 use Doctrine\Tests\OrmTestCase;
+use PHPUnit\Framework\Attributes\Group;
 
-/** @group DDC-659 */
+#[Group('DDC-659')]
 class ClassMetadataBuilderTest extends OrmTestCase
 {
-    /** @var ClassMetadata */
-    private $cm;
-    /** @var ClassMetadataBuilder */
-    private $builder;
+    private ClassMetadata $cm;
+    private ClassMetadataBuilder $builder;
 
     protected function setUp(): void
     {
@@ -50,14 +56,14 @@ class ClassMetadataBuilderTest extends OrmTestCase
 
         self::assertEquals(
             [
-                'name' => [
+                'name' => EmbeddedClassMapping::fromMappingArray([
                     'class' => Name::class,
                     'columnPrefix' => null,
                     'declaredField' => null,
                     'originalField' => null,
-                ],
+                ]),
             ],
-            $this->cm->embeddedClasses
+            $this->cm->embeddedClasses,
         );
     }
 
@@ -67,20 +73,20 @@ class ClassMetadataBuilderTest extends OrmTestCase
             $this->builder->addEmbedded(
                 'name',
                 Name::class,
-                'nm_'
-            )
+                'nm_',
+            ),
         );
 
         self::assertEquals(
             [
-                'name' => [
+                'name' => EmbeddedClassMapping::fromMappingArray([
                     'class' => Name::class,
                     'columnPrefix' => 'nm_',
                     'declaredField' => null,
                     'originalField' => null,
-                ],
+                ]),
             ],
-            $this->cm->embeddedClasses
+            $this->cm->embeddedClasses,
         );
     }
 
@@ -93,13 +99,13 @@ class ClassMetadataBuilderTest extends OrmTestCase
 
         $this->assertIsFluent($embeddedBuilder->build());
         self::assertEquals(
-            [
+            EmbeddedClassMapping::fromMappingArray([
                 'class' => Name::class,
                 'columnPrefix' => null,
                 'declaredField' => null,
                 'originalField' => null,
-            ],
-            $this->cm->embeddedClasses['name']
+            ]),
+            $this->cm->embeddedClasses['name'],
         );
     }
 
@@ -112,13 +118,13 @@ class ClassMetadataBuilderTest extends OrmTestCase
         $this->assertIsFluent($embeddedBuilder->build());
 
         self::assertEquals(
-            [
+            EmbeddedClassMapping::fromMappingArray([
                 'class' => Name::class,
                 'columnPrefix' => 'nm_',
                 'declaredField' => null,
                 'originalField' => null,
-            ],
-            $this->cm->embeddedClasses['name']
+            ]),
+            $this->cm->embeddedClasses['name'],
         );
     }
 
@@ -164,7 +170,7 @@ class ClassMetadataBuilderTest extends OrmTestCase
                 'indexes' => ['users_idx' => ['columns' => ['username', 'name']]],
                 'uniqueConstraints' => ['users_idx' => ['columns' => ['username', 'name']]],
             ],
-            $this->cm->table
+            $this->cm->table,
         );
     }
 
@@ -182,8 +188,16 @@ class ClassMetadataBuilderTest extends OrmTestCase
 
     public function testSetDiscriminatorColumn(): void
     {
-        $this->assertIsFluent($this->builder->setDiscriminatorColumn('discr', 'string', '124', null, null));
-        self::assertEquals(['fieldName' => 'discr', 'name' => 'discr', 'type' => 'string', 'length' => '124', 'columnDefinition' => null, 'enumType' => null, 'options' => []], $this->cm->discriminatorColumn);
+        $this->assertIsFluent($this->builder->setDiscriminatorColumn('discr', 'string', 124, null, null));
+        self::assertEquals(DiscriminatorColumnMapping::fromMappingArray([
+            'fieldName' => 'discr',
+            'name' => 'discr',
+            'type' => 'string',
+            'length' => 124,
+            'columnDefinition' => null,
+            'enumType' => null,
+            'options' => [],
+        ]), $this->cm->discriminatorColumn);
     }
 
     public function testAddDiscriminatorMapClass(): void
@@ -193,7 +207,7 @@ class ClassMetadataBuilderTest extends OrmTestCase
 
         self::assertEquals(
             ['test' => CmsUser::class, 'test2' => CmsGroup::class],
-            $this->cm->discriminatorMap
+            $this->cm->discriminatorMap,
         );
         self::assertEquals('test', $this->cm->discriminatorValue);
     }
@@ -204,16 +218,13 @@ class ClassMetadataBuilderTest extends OrmTestCase
         self::assertEquals(ClassMetadata::CHANGETRACKING_DEFERRED_EXPLICIT, $this->cm->changeTrackingPolicy);
     }
 
-    public function testChangeTrackingPolicyNotify(): void
-    {
-        $this->assertIsFluent($this->builder->setChangeTrackingPolicyNotify());
-        self::assertEquals(ClassMetadata::CHANGETRACKING_NOTIFY, $this->cm->changeTrackingPolicy);
-    }
-
     public function testAddField(): void
     {
         $this->assertIsFluent($this->builder->addField('name', 'string'));
-        self::assertEquals(['columnName' => 'name', 'fieldName' => 'name', 'type' => 'string'], $this->cm->fieldMappings['name']);
+        $mapping = $this->cm->getFieldMapping('name');
+        self::assertSame('name', $mapping->fieldName);
+        self::assertSame('name', $mapping->columnName);
+        self::assertSame('string', $mapping->type);
     }
 
     public function testCreateField(): void
@@ -223,14 +234,17 @@ class ClassMetadataBuilderTest extends OrmTestCase
 
         self::assertFalse(isset($this->cm->fieldMappings['name']));
         $this->assertIsFluent($fieldBuilder->build());
-        self::assertEquals(['columnName' => 'name', 'fieldName' => 'name', 'type' => 'string'], $this->cm->fieldMappings['name']);
+        $mapping = $this->cm->getFieldMapping('name');
+        self::assertSame('name', $mapping->fieldName);
+        self::assertSame('name', $mapping->columnName);
+        self::assertSame('string', $mapping->type);
     }
 
     public function testCreateVersionedField(): void
     {
         $this->builder->createField('name', 'integer')->columnName('username')->length(124)->nullable()->columnDefinition('foobar')->unique()->isVersionField()->build();
         self::assertEquals(
-            [
+            FieldMapping::fromMappingArray([
                 'columnDefinition' => 'foobar',
                 'columnName' => 'username',
                 'default' => 1,
@@ -239,8 +253,8 @@ class ClassMetadataBuilderTest extends OrmTestCase
                 'type' => 'integer',
                 'nullable' => true,
                 'unique' => true,
-            ],
-            $this->cm->fieldMappings['name']
+            ]),
+            $this->cm->fieldMappings['name'],
         );
     }
 
@@ -249,17 +263,18 @@ class ClassMetadataBuilderTest extends OrmTestCase
         $this->builder->createField('id', 'integer')->makePrimaryKey()->generatedValue()->build();
 
         self::assertEquals(['id'], $this->cm->identifier);
-        self::assertEquals(['columnName' => 'id', 'fieldName' => 'id', 'id' => true, 'type' => 'integer'], $this->cm->fieldMappings['id']);
+        self::assertEquals(FieldMapping::fromMappingArray(
+            ['columnName' => 'id', 'fieldName' => 'id', 'id' => true, 'type' => 'integer'],
+        ), $this->cm->fieldMappings['id']);
     }
 
     public function testCreateUnsignedOptionField(): void
     {
         $this->builder->createField('state', 'integer')->option('unsigned', true)->build();
 
-        self::assertEquals(
+        self::assertEquals(FieldMapping::fromMappingArray(
             ['fieldName' => 'state', 'type' => 'integer', 'options' => ['unsigned' => true], 'columnName' => 'state'],
-            $this->cm->fieldMappings['state']
-        );
+        ), $this->cm->fieldMappings['state']);
     }
 
     public function testAddLifecycleEvent(): void
@@ -276,20 +291,19 @@ class ClassMetadataBuilderTest extends OrmTestCase
                               ->addJoinColumn('group_id', 'id', true, false, 'CASCADE')
                               ->cascadeAll()
                               ->fetchExtraLazy()
-                              ->build()
+                              ->build(),
         );
 
         self::assertEquals(
             [
-                'groups' => [
+                'groups' => ManyToOneAssociationMapping::fromMappingArray([
                     'fieldName' => 'groups',
                     'targetEntity' => CmsGroup::class,
                     'cascade' => [
                         0 => 'remove',
                         1 => 'persist',
                         2 => 'refresh',
-                        3 => 'merge',
-                        4 => 'detach',
+                        3 => 'detach',
                     ],
                     'fetch' => 4,
                     'joinColumns' => [
@@ -304,15 +318,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                         ],
                     ],
                     'type' => 2,
-                    'mappedBy' => null,
                     'inversedBy' => null,
                     'isOwningSide' => true,
                     'sourceEntity' => CmsUser::class,
-                    'isCascadeRemove' => true,
-                    'isCascadePersist' => true,
-                    'isCascadeRefresh' => true,
-                    'isCascadeMerge' => true,
-                    'isCascadeDetach' => true,
                     'sourceToTargetKeyColumns' =>
                     ['group_id' => 'id'],
                     'joinColumnFieldNames' =>
@@ -320,9 +328,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                     'targetToSourceKeyColumns' =>
                     ['id' => 'group_id'],
                     'orphanRemoval' => false,
-                ],
+                ]),
             ],
-            $this->cm->associationMappings
+            $this->cm->associationMappings,
         );
     }
 
@@ -336,20 +344,19 @@ class ClassMetadataBuilderTest extends OrmTestCase
                 ->cascadeAll()
                 ->fetchExtraLazy()
                 ->makePrimaryKey()
-                ->build()
+                ->build(),
         );
 
         self::assertEquals(
             [
-                'groups' => [
+                'groups' => ManyToOneAssociationMapping::fromMappingArray([
                     'fieldName' => 'groups',
                     'targetEntity' => CmsGroup::class,
                     'cascade' => [
                         0 => 'remove',
                         1 => 'persist',
                         2 => 'refresh',
-                        3 => 'merge',
-                        4 => 'detach',
+                        3 => 'detach',
                     ],
                     'fetch' => 4,
                     'joinColumns' => [
@@ -364,15 +371,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                             ],
                     ],
                     'type' => 2,
-                    'mappedBy' => null,
                     'inversedBy' => null,
                     'isOwningSide' => true,
                     'sourceEntity' => CmsUser::class,
-                    'isCascadeRemove' => true,
-                    'isCascadePersist' => true,
-                    'isCascadeRefresh' => true,
-                    'isCascadeMerge' => true,
-                    'isCascadeDetach' => true,
                     'sourceToTargetKeyColumns' =>
                         ['group_id' => 'id'],
                     'joinColumnFieldNames' =>
@@ -381,9 +382,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                         ['id' => 'group_id'],
                     'orphanRemoval' => false,
                     'id' => true,
-                ],
+                ]),
             ],
-            $this->cm->associationMappings
+            $this->cm->associationMappings,
         );
     }
 
@@ -394,20 +395,19 @@ class ClassMetadataBuilderTest extends OrmTestCase
                               ->addJoinColumn('group_id', 'id', true, false, 'CASCADE')
                               ->cascadeAll()
                               ->fetchExtraLazy()
-                              ->build()
+                              ->build(),
         );
 
         self::assertEquals(
             [
-                'groups' => [
+                'groups' => OneToOneOwningSideMapping::fromMappingArray([
                     'fieldName' => 'groups',
                     'targetEntity' => CmsGroup::class,
                     'cascade' => [
                         0 => 'remove',
                         1 => 'persist',
                         2 => 'refresh',
-                        3 => 'merge',
-                        4 => 'detach',
+                        3 => 'detach',
                     ],
                     'fetch' => 4,
                     'joinColumns' => [
@@ -422,15 +422,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                         ],
                     ],
                     'type' => 1,
-                    'mappedBy' => null,
                     'inversedBy' => null,
                     'isOwningSide' => true,
                     'sourceEntity' => CmsUser::class,
-                    'isCascadeRemove' => true,
-                    'isCascadePersist' => true,
-                    'isCascadeRefresh' => true,
-                    'isCascadeMerge' => true,
-                    'isCascadeDetach' => true,
                     'sourceToTargetKeyColumns' =>
                     ['group_id' => 'id'],
                     'joinColumnFieldNames' =>
@@ -438,9 +432,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                     'targetToSourceKeyColumns' =>
                     ['id' => 'group_id'],
                     'orphanRemoval' => false,
-                ],
+                ]),
             ],
-            $this->cm->associationMappings
+            $this->cm->associationMappings,
         );
     }
 
@@ -454,20 +448,19 @@ class ClassMetadataBuilderTest extends OrmTestCase
                 ->cascadeAll()
                 ->fetchExtraLazy()
                 ->makePrimaryKey()
-                ->build()
+                ->build(),
         );
 
         self::assertEquals(
             [
-                'groups' => [
+                'groups' => OneToOneOwningSideMapping::fromMappingArray([
                     'fieldName' => 'groups',
                     'targetEntity' => CmsGroup::class,
                     'cascade' => [
                         0 => 'remove',
                         1 => 'persist',
                         2 => 'refresh',
-                        3 => 'merge',
-                        4 => 'detach',
+                        3 => 'detach',
                     ],
                     'fetch' => 4,
                     'id' => true,
@@ -483,15 +476,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                             ],
                     ],
                     'type' => 1,
-                    'mappedBy' => null,
                     'inversedBy' => null,
                     'isOwningSide' => true,
                     'sourceEntity' => CmsUser::class,
-                    'isCascadeRemove' => true,
-                    'isCascadePersist' => true,
-                    'isCascadeRefresh' => true,
-                    'isCascadeMerge' => true,
-                    'isCascadeDetach' => true,
                     'sourceToTargetKeyColumns' =>
                         ['group_id' => 'id'],
                     'joinColumnFieldNames' =>
@@ -499,9 +486,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                     'targetToSourceKeyColumns' =>
                         ['id' => 'group_id'],
                     'orphanRemoval' => false,
-                ],
+                ]),
             ],
-            $this->cm->associationMappings
+            $this->cm->associationMappings,
         );
     }
 
@@ -527,13 +514,13 @@ class ClassMetadataBuilderTest extends OrmTestCase
                               ->addInverseJoinColumn('user_id', 'id')
                               ->cascadeAll()
                               ->fetchExtraLazy()
-                              ->build()
+                              ->build(),
         );
 
         self::assertEquals(
             [
                 'groups' =>
-                [
+                ManyToManyOwningSideMapping::fromMappingArray([
                     'fieldName' => 'groups',
                     'targetEntity' => CmsGroup::class,
                     'cascade' =>
@@ -541,8 +528,7 @@ class ClassMetadataBuilderTest extends OrmTestCase
                         0 => 'remove',
                         1 => 'persist',
                         2 => 'refresh',
-                        3 => 'merge',
-                        4 => 'detach',
+                        3 => 'detach',
                     ],
                     'fetch' => 4,
                     'joinTable' =>
@@ -574,16 +560,10 @@ class ClassMetadataBuilderTest extends OrmTestCase
                         'name' => 'groups_users',
                     ],
                     'type' => 8,
-                    'mappedBy' => null,
                     'inversedBy' => null,
                     'isOwningSide' => true,
-                    'sourceEntity' => CmsUser::class,
-                    'isCascadeRemove' => true,
-                    'isCascadePersist' => true,
-                    'isCascadeRefresh' => true,
-                    'isCascadeMerge' => true,
-                    'isCascadeDetach' => true,
                     'isOnDeleteCascade' => true,
+                    'sourceEntity' => CmsUser::class,
                     'relationToSourceKeyColumns' =>
                     ['group_id' => 'id'],
                     'joinTableColumns' =>
@@ -594,9 +574,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                     'relationToTargetKeyColumns' =>
                     ['user_id' => 'id'],
                     'orphanRemoval' => false,
-                ],
+                ]),
             ],
-            $this->cm->associationMappings
+            $this->cm->associationMappings,
         );
     }
 
@@ -621,35 +601,27 @@ class ClassMetadataBuilderTest extends OrmTestCase
                         ->mappedBy('test')
                         ->setOrderBy(['test'])
                         ->setIndexBy('test')
-                        ->build()
+                        ->build(),
         );
 
         self::assertEquals(
             [
                 'groups' =>
-                [
+                OneToManyAssociationMapping::fromMappingArray([
                     'fieldName' => 'groups',
                     'targetEntity' => CmsGroup::class,
                     'mappedBy' => 'test',
-                    'orderBy' =>
-                [0 => 'test'],
+                    'orderBy' => [0 => 'test'],
                     'indexBy' => 'test',
                     'type' => 4,
-                    'inversedBy' => null,
                     'isOwningSide' => false,
                     'sourceEntity' => CmsUser::class,
                     'fetch' => 2,
-                    'cascade' =>
-                [],
-                    'isCascadeRemove' => false,
-                    'isCascadePersist' => false,
-                    'isCascadeRefresh' => false,
-                    'isCascadeMerge' => false,
-                    'isCascadeDetach' => false,
+                    'cascade' => [],
                     'orphanRemoval' => false,
-                ],
+                ]),
             ],
-            $this->cm->associationMappings
+            $this->cm->associationMappings,
         );
     }
 
@@ -672,12 +644,12 @@ class ClassMetadataBuilderTest extends OrmTestCase
                 ->createOneToOne('groups', CmsGroup::class)
                 ->addJoinColumn('group_id', 'id', true, false, 'CASCADE')
                 ->orphanRemoval()
-                ->build()
+                ->build(),
         );
 
         self::assertEquals(
             [
-                'groups' => [
+                'groups' => OneToOneOwningSideMapping::fromMappingArray([
                     'fieldName' => 'groups',
                     'targetEntity' => CmsGroup::class,
                     'cascade' => [],
@@ -694,15 +666,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                         ],
                     ],
                     'type' => 1,
-                    'mappedBy' => null,
                     'inversedBy' => null,
                     'isOwningSide' => true,
                     'sourceEntity' => CmsUser::class,
-                    'isCascadeRemove' => true,
-                    'isCascadePersist' => false,
-                    'isCascadeRefresh' => false,
-                    'isCascadeMerge' => false,
-                    'isCascadeDetach' => false,
                     'sourceToTargetKeyColumns' =>
                     ['group_id' => 'id'],
                     'joinColumnFieldNames' =>
@@ -710,9 +676,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                     'targetToSourceKeyColumns' =>
                     ['id' => 'group_id'],
                     'orphanRemoval' => true,
-                ],
+                ]),
             ],
-            $this->cm->associationMappings
+            $this->cm->associationMappings,
         );
     }
 
@@ -723,31 +689,25 @@ class ClassMetadataBuilderTest extends OrmTestCase
                 ->createOneToMany('groups', CmsGroup::class)
                 ->mappedBy('test')
                 ->orphanRemoval()
-                ->build()
+                ->build(),
         );
 
         self::assertEquals(
             [
                 'groups' =>
-                [
+                OneToManyAssociationMapping::fromMappingArray([
                     'fieldName' => 'groups',
                     'targetEntity' => CmsGroup::class,
                     'mappedBy' => 'test',
                     'type' => 4,
-                    'inversedBy' => null,
                     'isOwningSide' => false,
                     'sourceEntity' => CmsUser::class,
                     'fetch' => 2,
                     'cascade' => [],
-                    'isCascadeRemove' => true,
-                    'isCascadePersist' => false,
-                    'isCascadeRefresh' => false,
-                    'isCascadeMerge' => false,
-                    'isCascadeDetach' => false,
                     'orphanRemoval' => true,
-                ],
+                ]),
             ],
-            $this->cm->associationMappings
+            $this->cm->associationMappings,
         );
     }
 
@@ -772,7 +732,7 @@ class ClassMetadataBuilderTest extends OrmTestCase
 
         self::assertEquals(
             [
-                'groups' => [
+                'groups' => ManyToManyOwningSideMapping::fromMappingArray([
                     'fieldName' => 'groups',
                     'targetEntity' => CmsGroup::class,
                     'cascade' => [],
@@ -798,15 +758,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                         'name' => 'cmsuser_cmsgroup',
                     ],
                     'type' => 8,
-                    'mappedBy' => null,
                     'inversedBy' => null,
                     'isOwningSide' => true,
                     'sourceEntity' => CmsUser::class,
-                    'isCascadeRemove' => false,
-                    'isCascadePersist' => false,
-                    'isCascadeRefresh' => false,
-                    'isCascadeMerge' => false,
-                    'isCascadeDetach' => false,
                     'isOnDeleteCascade' => true,
                     'relationToSourceKeyColumns' => ['group_id' => 'id'],
                     'joinTableColumns' => [
@@ -815,9 +769,9 @@ class ClassMetadataBuilderTest extends OrmTestCase
                     ],
                     'relationToTargetKeyColumns' => ['cmsgroup_id' => 'id'],
                     'orphanRemoval' => true,
-                ],
+                ]),
             ],
-            $this->cm->associationMappings
+            $this->cm->associationMappings,
         );
     }
 
