@@ -5,12 +5,14 @@ declare(strict_types=1);
 namespace Doctrine\Tests\ORM\Mapping;
 
 use Doctrine\DBAL\Platforms\AbstractPlatform;
+use Doctrine\DBAL\Schema\Name\UnquotedIdentifierFolding;
 use Doctrine\ORM\Mapping\DefaultQuoteStrategy;
 use Doctrine\Tests\Models\NonPublicSchemaJoins\User as NonPublicSchemaUser;
 use Doctrine\Tests\OrmTestCase;
 use PHPUnit\Framework\Attributes\Group;
 
 use function assert;
+use function enum_exists;
 
 /**
  * Doctrine\Tests\ORM\Mapping\DefaultQuoteStrategyTest
@@ -24,7 +26,7 @@ class DefaultQuoteStrategyTest extends OrmTestCase
         $em       = $this->getTestEntityManager();
         $metadata = $em->getClassMetadata(NonPublicSchemaUser::class);
         $strategy = new DefaultQuoteStrategy();
-        $platform = $this->getMockForAbstractClass(AbstractPlatform::class);
+        $platform = $this->getMockForAbstractClass(AbstractPlatform::class, enum_exists(UnquotedIdentifierFolding::class) ? [UnquotedIdentifierFolding::UPPER] : []);
         assert($platform instanceof AbstractPlatform);
 
         self::assertSame(
