@@ -10,9 +10,6 @@ use Doctrine\Tests\Models\Enums\FaultySwitch;
 use Doctrine\Tests\OrmTestCase;
 use ReflectionClass;
 
-/**
- * @requires PHP >= 8.1
- */
 class TypedEnumFieldMapperTest extends OrmTestCase
 {
     private static function defaultTypedFieldMapper(): DefaultTypedFieldMapper
@@ -26,7 +23,7 @@ class TypedEnumFieldMapperTest extends OrmTestCase
 
         $this->expectException(MappingException::class);
         $this->expectExceptionMessage(
-            'Attempting to map a non-backed enum type Doctrine\Tests\Models\Enums\SwitchStatus in entity Doctrine\Tests\Models\Enums\FaultySwitch::$status. Please use backed enums only'
+            'Attempting to map a non-backed enum type Doctrine\Tests\Models\Enums\SwitchStatus in entity Doctrine\Tests\Models\Enums\FaultySwitch::$status. Please use backed enums only',
         );
 
         self::defaultTypedFieldMapper()->validateAndComplete(['fieldName' => 'status'], $reflectionClass->getProperty('status'));
