@@ -4,6 +4,7 @@ declare(strict_types=1);
 
 namespace Doctrine\Tests\ORM\Query;
 
+use Doctrine\ORM\AbstractQuery;
 use Doctrine\ORM\EntityManagerInterface;
 use Doctrine\ORM\Mapping\Column;
 use Doctrine\ORM\Mapping\Entity;
@@ -11,15 +12,19 @@ use Doctrine\ORM\Mapping\GeneratedValue;
 use Doctrine\ORM\Mapping\Id;
 use Doctrine\ORM\Mapping\OneToOne;
 use Doctrine\ORM\Query;
+use Doctrine\ORM\Query\AST\Functions\ConcatFunction;
+use Doctrine\ORM\Query\Parser;
 use Doctrine\ORM\Query\ParserResult;
 use Doctrine\ORM\Query\QueryException;
 use Doctrine\Tests\Mocks\NullSqlWalker;
 use Doctrine\Tests\OrmTestCase;
+use PHPUnit\Framework\Attributes\DataProvider;
+use PHPUnit\Framework\Attributes\Group;
 
 class LanguageRecognitionTest extends OrmTestCase
 {
-    /** @var EntityManagerInterface */
-    private $entityManager;
+    private EntityManagerInterface $entityManager;
+    private int $hydrationMode = AbstractQuery::HYDRATE_OBJECT;
 
     protected function setUp(): void
     {
@@ -43,12 +48,13 @@ class LanguageRecognitionTest extends OrmTestCase
     {
         $query = $this->entityManager->createQuery($dql);
         $query->setDQL($dql);
+        $query->setHydrationMode($this->hydrationMode);
 
         foreach ($hints as $key => $value) {
             $query->setHint($key, $value);
         }
 
-        $parser = new Query\Parser($query);
+        $parser = new Parser($query);
 
         // We do NOT test SQL output here. That only unnecessarily slows down the tests!
         $query->setHint(Query::HINT_CUSTOM_OUTPUT_WALKER, NullSqlWalker::class);
@@ -71,7 +77,7 @@ class LanguageRecognitionTest extends OrmTestCase
         $this->assertValidDQL('SELECT u FROM Doctrine\Tests\Models\CMS\CmsUser u');
     }
 
-    /** @dataProvider invalidDQL */
+    #[DataProvider('invalidDQL')]
     public function testRejectsInvalidDQL(string $dql): void
     {
         $this->expectException(QueryException::class);
@@ -80,7 +86,7 @@ class LanguageRecognitionTest extends OrmTestCase
             [
                 'Unknown' => 'Unknown',
                 'CMS' => 'Doctrine\Tests\Models\CMS',
-            ]
+            ],
         );
 
         $this->parseDql($dql);
@@ -96,8 +102,6 @@ class LanguageRecognitionTest extends OrmTestCase
             ['SELECT \foo FROM Doctrine\Tests\Models\CMS\CmsUser \foo'],
             ['SELECT foo\ FROM Doctrine\Tests\Models\CMS\CmsUser foo\\'],
             ['SELECT foo\bar FROM Doctrine\Tests\Models\CMS\CmsUser foo\bar'],
-            ['SELECT foo:bar FROM Doctrine\Tests\Models\CMS\CmsUser foo:bar'],
-            ['SELECT foo: FROM Doctrine\Tests\Models\CMS\CmsUser foo:'],
 
             /* Checks for invalid AbstractSchemaName */
             ['SELECT u FROM UnknownClass u'],  // unknown
@@ -105,23 +109,13 @@ class LanguageRecognitionTest extends OrmTestCase
             ['SELECT u FROM \Unknown\Class u'], // unknown, leading backslash
             ['SELECT u FROM Unknown\\\\Class u'], // unknown, syntactically bogus (duplicate \\)
             ['SELECT u FROM Unknown\Class\ u'], // unknown, syntactically bogus (trailing \)
-            ['SELECT u FROM Unknown:Class u'], // unknown, with namespace alias
-            ['SELECT u FROM Unknown::Class u'], // unknown, with PAAMAYIM_NEKUDOTAYIM
-            ['SELECT u FROM Unknown:Class:Name u'], // unknown, with invalid namespace alias
-            ['SELECT u FROM UnknownClass: u'], // unknown, with invalid namespace alias
-            ['SELECT u FROM Unknown:Class: u'], // unknown, with invalid namespace alias
             ['SELECT u FROM Doctrine\Tests\Models\CMS\\\\CmsUser u'], // syntactically bogus (duplicate \\)array('SELECT u FROM Doctrine\Tests\Models\CMS\CmsUser\ u'), // syntactically bogus (trailing \)
-            ['SELECT u FROM CMS::User u'],
-            ['SELECT u FROM CMS:User: u'],
-            ['SELECT u FROM CMS:User:Foo u'],
 
             /* Checks for invalid AliasResultVariable */
             ['SELECT \'foo\' AS \foo FROM Doctrine\Tests\Models\CMS\CmsUser u'],
             ['SELECT \'foo\' AS \foo\bar FROM Doctrine\Tests\Models\CMS\CmsUser u'],
             ['SELECT \'foo\' AS foo\ FROM Doctrine\Tests\Models\CMS\CmsUser u'],
             ['SELECT \'foo\' AS foo\\\\bar FROM Doctrine\Tests\Models\CMS\CmsUser u'],
-            ['SELECT \'foo\' AS foo: FROM Doctrine\Tests\Models\CMS\CmsUser u'],
-            ['SELECT \'foo\' AS foo:bar FROM Doctrine\Tests\Models\CMS\CmsUser u'],
 
             ['0'],
         ];
@@ -273,13 +267,13 @@ class LanguageRecognitionTest extends OrmTestCase
         $this->assertValidDQL('SELECT u.name FROM Doctrine\Tests\Models\CMS\CmsUser u JOIN Doctrine\Tests\Models\CMS\CmsArticle a WITH a.user = u.id');
     }
 
-    /** @group DDC-3701 */
+    #[Group('DDC-3701')]
     public function testJoinClassPathUsingWHERE(): void
     {
         $this->assertValidDQL('SELECT u.name FROM Doctrine\Tests\Models\CMS\CmsUser u JOIN Doctrine\Tests\Models\CMS\CmsArticle a WHERE a.user = u.id');
     }
 
-    /** @group DDC-3701 */
+    #[Group('DDC-3701')]
     public function testDDC3701WHEREIsNotWITH(): void
     {
         $this->assertInvalidDQL('SELECT c FROM Doctrine\Tests\Models\Company\CompanyContract c JOIN Doctrine\Tests\Models\Company\CompanyEmployee e WHERE e.id = c.salesPerson WHERE c.completed = true');
@@ -330,14 +324,14 @@ class LanguageRecognitionTest extends OrmTestCase
         $this->assertValidDQL("SELECT (SELECT (SUM(u.id) / COUNT(u.id)) FROM Doctrine\Tests\Models\CMS\CmsUser u2) value FROM Doctrine\Tests\Models\CMS\CmsUser u WHERE u.name = 'jon'");
     }
 
-    /** @group DDC-1079 */
+    #[Group('DDC-1079')]
     public function testSelectLiteralInSubselect(): void
     {
         $this->assertValidDQL('SELECT (SELECT 1 FROM Doctrine\Tests\Models\CMS\CmsUser u2) value FROM Doctrine\Tests\Models\CMS\CmsUser u');
         $this->assertValidDQL('SELECT (SELECT 0 FROM Doctrine\Tests\Models\CMS\CmsUser u2) value FROM Doctrine\Tests\Models\CMS\CmsUser u');
     }
 
-    /** @group DDC-1077 */
+    #[Group('DDC-1077')]
     public function testConstantValueInSelect(): void
     {
         $this->assertValidDQL("SELECT u.name, 'foo' AS bar FROM Doctrine\Tests\Models\CMS\CmsUser u");
@@ -557,21 +551,21 @@ class LanguageRecognitionTest extends OrmTestCase
         $this->assertValidDQL('SELECT u, u.id + ?1 AS someNumber FROM Doctrine\Tests\Models\CMS\CmsUser u');
     }
 
-    /** @group DDC-1091 */
+    #[Group('DDC-1091')]
     public function testCustomFunctionsReturningStringInStringPrimary(): void
     {
-        $this->entityManager->getConfiguration()->addCustomStringFunction('CC', Query\AST\Functions\ConcatFunction::class);
+        $this->entityManager->getConfiguration()->addCustomStringFunction('CC', ConcatFunction::class);
 
         $this->assertValidDQL("SELECT u FROM Doctrine\Tests\Models\CMS\CmsUser u WHERE CC('%', u.name) LIKE '%foo%'");
     }
 
-    /** @group DDC-505 */
+    #[Group('DDC-505')]
     public function testDQLKeywordInJoinIsAllowed(): void
     {
         $this->assertValidDQL('SELECT u FROM ' . __NAMESPACE__ . '\DQLKeywordsModelUser u JOIN u.group g');
     }
 
-    /** @group DDC-505 */
+    #[Group('DDC-505')]
     public function testDQLKeywordInConditionIsAllowed(): void
     {
         $this->assertValidDQL('SELECT g FROM ' . __NAMESPACE__ . '\DQLKeywordsModelGroup g WHERE g.from=0');
@@ -583,43 +577,43 @@ class LanguageRecognitionTest extends OrmTestCase
         $this->assertInvalidDQL('SELECT u.id FROM Doctrine\Tests\Models\CMS\CmsUser u WHERE u.address = ?1');
     }
 
-    /** @group DDC-617 */
+    #[Group('DDC-617')]
     public function testSelectOnlyNonRootEntityAlias(): void
     {
         $this->assertInvalidDQL('SELECT g FROM Doctrine\Tests\Models\CMS\CmsUser u JOIN u.groups g');
     }
 
-    /** @group DDC-1108 */
+    #[Group('DDC-1108')]
     public function testInputParameterSingleChar(): void
     {
         $this->assertValidDQL('SELECT u FROM Doctrine\Tests\Models\CMS\CmsUser u WHERE u.name = :q');
     }
 
-    /** @group DDC-1053 */
+    #[Group('DDC-1053')]
     public function testGroupBy(): void
     {
         $this->assertValidDQL('SELECT g.id, count(u.id) FROM Doctrine\Tests\Models\CMS\CmsGroup g JOIN g.users u GROUP BY g.id');
     }
 
-    /** @group DDC-1053 */
+    #[Group('DDC-1053')]
     public function testGroupByIdentificationVariable(): void
     {
         $this->assertValidDQL('SELECT g, count(u.id) FROM Doctrine\Tests\Models\CMS\CmsGroup g JOIN g.users u GROUP BY g');
     }
 
-    /** @group DDC-1053 */
+    #[Group('DDC-1053')]
     public function testGroupByUnknownIdentificationVariable(): void
     {
         $this->assertInvalidDQL('SELECT g, count(u.id) FROM Doctrine\Tests\Models\CMS\CmsGroup g JOIN g.users u GROUP BY m');
     }
 
-    /** @group DDC-117 */
+    #[Group('DDC-117')]
     public function testSizeOfForeignKeyOneToManyPrimaryKeyEntity(): void
     {
         $this->assertValidDQL('SELECT a, t FROM Doctrine\Tests\Models\DDC117\DDC117Article a JOIN a.translations t WHERE SIZE(a.translations) > 0');
     }
 
-    /** @group DDC-117 */
+    #[Group('DDC-117')]
     public function testSizeOfForeignKeyManyToManyPrimaryKeyEntity(): void
     {
         $this->assertValidDQL('SELECT e, t FROM Doctrine\Tests\Models\DDC117\DDC117Editor e JOIN e.reviewingTranslations t WHERE SIZE(e.reviewingTranslations) > 0');
@@ -635,31 +629,31 @@ class LanguageRecognitionTest extends OrmTestCase
         $this->assertValidDQL("select COALESCE(NULLIF(u.name, ''), u.username) as Display FROM Doctrine\Tests\Models\CMS\CmsUser u");
     }
 
-    /** @group DDC-1858 */
+    #[Group('DDC-1858')]
     public function testHavingSupportIsNullExpression(): void
     {
         $this->assertValidDQL('SELECT u.name FROM Doctrine\Tests\Models\CMS\CmsUser u HAVING u.username IS NULL');
     }
 
-    /** @group DDC-3085 */
+    #[Group('DDC-3085')]
     public function testHavingSupportResultVariableInNullComparisonExpression(): void
     {
         $this->assertValidDQL('SELECT u AS user, SUM(a.id) AS score FROM Doctrine\Tests\Models\CMS\CmsUser u LEFT JOIN Doctrine\Tests\Models\CMS\CmsAddress a WITH a.user = u GROUP BY u HAVING score IS NOT NULL AND score >= 5');
     }
 
-    /** @group DDC-1858 */
+    #[Group('DDC-1858')]
     public function testHavingSupportLikeExpression(): void
     {
         $this->assertValidDQL("SELECT _u.id, count(_articles) as uuuu FROM Doctrine\Tests\Models\CMS\CmsUser _u LEFT JOIN _u.articles _articles GROUP BY _u HAVING uuuu LIKE '3'");
     }
 
-    /** @group DDC-3018 */
+    #[Group('DDC-3018')]
     public function testNewLiteralExpression(): void
     {
         $this->assertValidDQL('SELECT new ' . __NAMESPACE__ . "\\DummyStruct(u.id, 'foo', 1, true) FROM Doctrine\Tests\Models\CMS\CmsUser u");
     }
 
-    /** @group DDC-3075 */
+    #[Group('DDC-3075')]
     public function testNewLiteralWithSubselectExpression(): void
     {
         $this->assertValidDQL('SELECT new ' . __NAMESPACE__ . "\\DummyStruct(u.id, 'foo', (SELECT 1 FROM Doctrine\Tests\Models\CMS\CmsUser su), true) FROM Doctrine\Tests\Models\CMS\CmsUser u");
@@ -668,45 +662,33 @@ class LanguageRecognitionTest extends OrmTestCase
     public function testStringPrimaryAcceptsAggregateExpression(): void
     {
         $this->assertValidDQL(
-            'SELECT CONCAT(a.topic, MAX(a.version)) last FROM Doctrine\Tests\Models\CMS\CmsArticle a GROUP BY a'
+            'SELECT CONCAT(a.topic, MAX(a.version)) last FROM Doctrine\Tests\Models\CMS\CmsArticle a GROUP BY a',
         );
     }
 }
 
-/** @Entity */
+#[Entity]
 class DQLKeywordsModelUser
 {
-    /**
-     * @var int
-     * @Id
-     * @Column(type="integer")
-     * @GeneratedValue
-     */
-    private $id;
+    #[Id]
+    #[Column(type: 'integer')]
+    #[GeneratedValue]
+    private int $id;
 
-    /**
-     * @var DQLKeywordsModelGroup
-     * @OneToOne(targetEntity="DQLKeywordsModelGroup")
-     */
-    private $group;
+    #[OneToOne(targetEntity: 'DQLKeywordsModelGroup')]
+    private DQLKeywordsModelGroup $group;
 }
 
-/** @Entity */
+#[Entity]
 class DQLKeywordsModelGroup
 {
-    /**
-     * @var int
-     * @Id
-     * @Column(type="integer")
-     * @GeneratedValue
-     */
-    private $id;
+    #[Id]
+    #[Column(type: 'integer')]
+    #[GeneratedValue]
+    private int $id;
 
-    /**
-     * @var string
-     * @Column
-     */
-    private $from;
+    #[Column]
+    private string $from;
 }
 
 class DummyStruct
