@@ -4,23 +4,20 @@ declare(strict_types=1);
 
 namespace Doctrine\Tests\ORM\Query;
 
-use Doctrine\Common\Persistence\PersistentObject;
 use Doctrine\ORM\Query;
 use Doctrine\ORM\Query\Parser;
 use Doctrine\ORM\Query\QueryException;
 use Doctrine\ORM\Query\TokenType;
 use Doctrine\Tests\Models\CMS\CmsUser;
 use Doctrine\Tests\OrmTestCase;
+use PHPUnit\Framework\Attributes\DataProvider;
+use PHPUnit\Framework\Attributes\DoesNotPerformAssertions;
+use PHPUnit\Framework\Attributes\Group;
 use stdClass;
 
-use function class_exists;
-
 class ParserTest extends OrmTestCase
 {
-    /**
-     * @covers \Doctrine\ORM\Query\Parser::AbstractSchemaName
-     * @group DDC-3715
-     */
+    #[Group('DDC-3715')]
     public function testAbstractSchemaNameSupportsFQCN(): void
     {
         $parser = $this->createParser(CmsUser::class);
@@ -28,10 +25,7 @@ class ParserTest extends OrmTestCase
         self::assertEquals(CmsUser::class, $parser->AbstractSchemaName());
     }
 
-    /**
-     * @covers Doctrine\ORM\Query\Parser::AbstractSchemaName
-     * @group DDC-3715
-     */
+    #[Group('DDC-3715')]
     public function testAbstractSchemaNameSupportsClassnamesWithLeadingBackslash(): void
     {
         $parser = $this->createParser('\\' . CmsUser::class);
@@ -39,10 +33,7 @@ class ParserTest extends OrmTestCase
         self::assertEquals('\\' . CmsUser::class, $parser->AbstractSchemaName());
     }
 
-    /**
-     * @covers \Doctrine\ORM\Query\Parser::AbstractSchemaName
-     * @group DDC-3715
-     */
+    #[Group('DDC-3715')]
     public function testAbstractSchemaNameSupportsIdentifier(): void
     {
         $parser = $this->createParser(stdClass::class);
@@ -50,60 +41,19 @@ class ParserTest extends OrmTestCase
         self::assertEquals(stdClass::class, $parser->AbstractSchemaName());
     }
 
-    /**
-     * @covers \Doctrine\ORM\Query\Parser::AbstractSchemaName
-     * @group DDC-3715
-     */
-    public function testAbstractSchemaNameSupportsNamespaceAlias(): void
-    {
-        if (! class_exists(PersistentObject::class)) {
-            self::markTestSkipped('This test requires doctrine/persistence 2');
-        }
-
-        $parser = $this->createParser('CMS:CmsUser');
-
-        $parser->getEntityManager()->getConfiguration()->addEntityNamespace('CMS', 'Doctrine\Tests\Models\CMS');
-
-        self::assertEquals(CmsUser::class, $parser->AbstractSchemaName());
-    }
-
-    /**
-     * @covers \Doctrine\ORM\Query\Parser::AbstractSchemaName
-     * @group DDC-3715
-     */
-    public function testAbstractSchemaNameSupportsNamespaceAliasWithRelativeClassname(): void
-    {
-        if (! class_exists(PersistentObject::class)) {
-            self::markTestSkipped('This test requires doctrine/persistence 2');
-        }
-
-        $parser = $this->createParser('Model:CMS\CmsUser');
-
-        $parser->getEntityManager()->getConfiguration()->addEntityNamespace('Model', 'Doctrine\Tests\Models');
-
-        self::assertEquals(CmsUser::class, $parser->AbstractSchemaName());
-    }
-
-    /**
-     * @dataProvider validMatches
-     * @covers Doctrine\ORM\Query\Parser::match
-     * @group DDC-3701
-     */
-    public function testMatch(int $expectedToken, string $inputString): void
+    #[DataProvider('validMatches')]
+    #[Group('DDC-3701')]
+    #[DoesNotPerformAssertions]
+    public function testMatch(TokenType $expectedToken, string $inputString): void
     {
         $parser = $this->createParser($inputString);
 
         $parser->match($expectedToken); // throws exception if not matched
-
-        $this->addToAssertionCount(1);
     }
 
-    /**
-     * @dataProvider invalidMatches
-     * @covers Doctrine\ORM\Query\Parser::match
-     * @group DDC-3701
-     */
-    public function testMatchFailure(int $expectedToken, string $inputString): void
+    #[DataProvider('invalidMatches')]
+    #[Group('DDC-3701')]
+    public function testMatchFailure(TokenType $expectedToken, string $inputString): void
     {
         $this->expectException(QueryException::class);
 
@@ -145,7 +95,6 @@ class ParserTest extends OrmTestCase
             // The following are qualified or aliased names and must not be accepted where only an Identifier is expected
             [TokenType::T_IDENTIFIER, '\\Some\\Class'],
             [TokenType::T_IDENTIFIER, 'Some\\Class'],
-            [TokenType::T_IDENTIFIER, 'Some:Name'],
         ];
     }
 
@@ -153,9 +102,8 @@ class ParserTest extends OrmTestCase
      * PHP 7.4 would fail with Notice: Trying to access array offset on value of type null.
      *
      * @see https://github.com/doctrine/orm/pull/7934
-     *
-     * @group GH7934
      */
+    #[Group('GH7934')]
     public function testNullLookahead(): void
     {
         $query = new Query($this->getTestEntityManager());
